@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -38,21 +38,25 @@ exception statement from your version. */
 
 package gnu.java.awt;
 
+import gnu.java.awt.EmbeddedWindow;
 import gnu.java.awt.peer.ClasspathFontPeer;
+import gnu.java.awt.peer.EmbeddedWindowPeer;
 import gnu.java.awt.peer.ClasspathTextLayoutPeer;
 
 import java.awt.AWTException;
-import java.awt.Image;
 import java.awt.Dimension;
 import java.awt.DisplayMode;
+import java.awt.EventQueue;
 import java.awt.Font;
 import java.awt.FontMetrics;
-import java.awt.GraphicsEnvironment;
 import java.awt.GraphicsDevice;
+import java.awt.GraphicsEnvironment;
+import java.awt.Image;
 import java.awt.Toolkit;
 import java.awt.font.FontRenderContext;
 import java.awt.image.ColorModel;
 import java.awt.image.ImageProducer;
+import java.awt.peer.RobotPeer;
 import java.io.File;
 import java.io.InputStream;
 import java.net.MalformedURLException;
@@ -60,8 +64,8 @@ import java.net.URL;
 import java.text.AttributedString;
 import java.util.HashMap;
 import java.util.Map;
-import java.awt.peer.RobotPeer;
 
+import javax.imageio.spi.IIORegistry;
 
 /**
  * An abstract superclass for Classpath toolkits.
@@ -92,7 +96,7 @@ public abstract class ClasspathToolkit
    * #getImage(java.net.URL)}. For images that were loaded via a path
    * to an image file, the map contains a key with a file URL.
    */
-  private Map imageCache;
+  private HashMap imageCache;
 
 
   /**
@@ -171,7 +175,6 @@ public abstract class ClasspathToolkit
    * this font peer should have, such as size, weight, family name, or
    * transformation.
    */
-
   public abstract ClasspathFontPeer getClasspathFontPeer (String name, Map attrs); 
 
   public abstract ClasspathTextLayoutPeer 
@@ -186,7 +189,6 @@ public abstract class ClasspathToolkit
    * implement {@link java.awt.font.OpenType} or
    * {@link java.awt.font.MultipleMaster}.
    */
-
   public Font getFont (String name, Map attrs) 
   {
     return new Font (name, attrs);
@@ -354,4 +356,24 @@ public abstract class ClasspathToolkit
 
   public abstract RobotPeer createRobot (GraphicsDevice screen)
     throws AWTException;
+
+  /**
+   * Creates an embedded window peer, and associates it with an
+   * EmbeddedWindow object.
+   *
+   * @param w The embedded window with which to associate a peer.
+   */
+  public abstract EmbeddedWindowPeer createEmbeddedWindow (EmbeddedWindow w);
+
+  /** 
+   * Used to register ImageIO SPIs provided by the toolkit.
+   */
+
+  public void registerImageIOSpis(IIORegistry reg)
+  {
+  }
+
+  public abstract boolean nativeQueueEmpty();
+  public abstract void wakeNativeQueue();  
+  public abstract void iterateNativeQueue(EventQueue locked, boolean block);
 }
