@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -63,8 +63,6 @@ public class GdkFontMetrics extends FontMetrics
   static final int TEXT_METRICS_X_ADVANCE = 4;
   static final int TEXT_METRICS_Y_ADVANCE = 5;
 
-  static native void getPeerFontMetrics(GdkFontPeer peer, double [] metrics);
-  static native void getPeerTextMetrics(GdkFontPeer peer, String str, double [] metrics);
 
   public GdkFontMetrics (Font font)
   {    
@@ -77,12 +75,7 @@ public class GdkFontMetrics extends FontMetrics
 
     font_metrics = new int[5];
     double [] hires = new double[5];
-
-    if (GtkToolkit.useGraphics2D ())
-      GdkGraphics2D.getPeerFontMetrics(peer, hires);
-    else
-      getPeerFontMetrics (peer, hires);
-
+    peer.getFontMetrics (hires);
     for (int i = 0; i < 5; ++i)
       font_metrics[i] = (int) hires[i];
   }
@@ -90,10 +83,7 @@ public class GdkFontMetrics extends FontMetrics
   public int stringWidth (String str)
   {
     double [] hires = new double[6];
-    if (GtkToolkit.useGraphics2D())
-      GdkGraphics2D.getPeerTextMetrics(peer, str, hires);
-    else
-      getPeerTextMetrics(peer, str, hires);
+    peer.getTextMetrics(str, hires);
     return (int) hires [TEXT_METRICS_WIDTH];
   }
 
@@ -115,7 +105,6 @@ public class GdkFontMetrics extends FontMetrics
   public int getLeading ()
   {
     return 1;
-//      return metrics[ASCENT] + metrics[DESCENT];
   }
 
   public int getAscent ()
