@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -63,6 +63,7 @@ import java.awt.event.ItemEvent;
 import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
 import java.awt.event.PaintEvent;
+import java.awt.event.TextEvent;
 import java.awt.image.BufferedImage;
 import java.awt.image.ColorModel;
 import java.awt.image.ImageObserver;
@@ -104,6 +105,13 @@ public class GtkComponentPeer extends GtkGenericPeer
 
   native boolean isRealized ();
 
+  void realize ()
+  {
+    // Default implementation does nothing
+  }
+
+  native void setNativeEventMask ();
+
   void create ()
   {
     throw new RuntimeException ();
@@ -136,6 +144,10 @@ public class GtkComponentPeer extends GtkGenericPeer
     if (awtComponent instanceof Window
         || (parent != null && ! parent.isShowing ()))
       setParentAndBounds ();
+
+    setNativeEventMask ();
+
+    realize ();
   }
 
   void setParentAndBounds ()
@@ -530,6 +542,11 @@ public class GtkComponentPeer extends GtkGenericPeer
 				item, stateChange));
   }
 
+  protected void postTextEvent ()
+  {
+    q().postEvent (new TextEvent (awtComponent, TextEvent.TEXT_VALUE_CHANGED));
+  }
+
   public GraphicsConfiguration getGraphicsConfiguration ()
   {
     // FIXME: just a stub for now.
