@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -99,6 +99,11 @@ public class GtkImage extends Image
    */
   boolean offScreen;
 
+  /**
+   * Error flag for loading.
+   */
+  boolean errorLoading;
+
   /**
    * Original source, if created from an ImageProducer.
    */
@@ -176,6 +181,7 @@ public class GtkImage extends Image
     isLoaded = false;
     observers = new Vector();
     source = producer;
+    errorLoading = false;
     source.startProduction(new GtkImageConsumer(this, source));
     offScreen = false;
   }
@@ -243,6 +249,13 @@ public class GtkImage extends Image
     this.width = width;
     this.height = height;
     props = (properties != null) ? properties : new Hashtable();
+
+    if (width <= 0 || height <= 0 || pixels == null)
+      {
+	errorLoading = true;
+	return;
+      }
+
     isLoaded = true;
     deliver();
     createPixmap();
@@ -346,7 +359,12 @@ public class GtkImage extends Image
   public int checkImage (ImageObserver observer)
   {
     if (addObserver(observer))
-      return 0;
+      {
+	if (errorLoading == true)
+	  return ImageObserver.ERROR;
+	else
+	  return 0;
+      }
 
     return ImageObserver.ALLBITS | ImageObserver.WIDTH | ImageObserver.HEIGHT;
   }
