@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -41,20 +41,51 @@ package gnu.java.awt.peer.gtk;
 import java.awt.Dimension;
 import java.awt.Font;
 import java.awt.FontMetrics;
+import java.awt.Rectangle;
 import java.awt.TextArea;
 import java.awt.peer.TextAreaPeer;
+import java.awt.peer.TextComponentPeer;
 
-public class GtkTextAreaPeer extends GtkTextComponentPeer
-  implements TextAreaPeer
+public class GtkTextAreaPeer extends GtkComponentPeer
+  implements TextComponentPeer, TextAreaPeer
 {
   private static transient int DEFAULT_ROWS = 10;
   private static transient int DEFAULT_COLS = 80;
 
   native void create (int width, int height, int scrollbarVisibility);
 
-  native void gtkSetFont (String name, int style, int size);
+  native void gtkWidgetModifyFont (String name, int style, int size);
   native void gtkWidgetRequestFocus ();
 
+  public native void connectSignals ();
+
+  public native int getCaretPosition ();
+  public native void setCaretPosition (int pos);
+  public native int getSelectionStart ();
+  public native int getSelectionEnd ();
+  public native String getText ();
+  public native void select (int start, int end);
+  public native void setEditable (boolean state);
+  public native void setText (String text);
+
+  public int getIndexAtPoint(int x, int y)
+  {
+    // FIXME
+    return 0;
+  }
+
+  public Rectangle getCharacterBounds (int pos)
+  {
+    // FIXME
+    return null;
+  }
+
+  public long filterEvents (long filter)
+  {
+    // FIXME
+    return filter;
+  }
+
   void create ()
   {
     Font f = awtComponent.getFont ();
@@ -68,11 +99,7 @@ public class GtkTextAreaPeer extends GtkTextComponentPeer
 	awtComponent.setFont (f);
       }
 
-    FontMetrics fm;
-    if (GtkToolkit.useGraphics2D ())
-      fm = new GdkClasspathFontPeerMetrics (f);
-    else
-      fm = new GdkFontMetrics (f);
+    FontMetrics fm = getFontMetrics (f);
 
     TextArea ta = ((TextArea) awtComponent);
     int sizeRows = ta.getRows ();
@@ -91,6 +118,9 @@ public class GtkTextAreaPeer extends GtkTextComponentPeer
   public GtkTextAreaPeer (TextArea ta)
   {
     super (ta);
+
+    setText (ta.getText ());
+    setCaretPosition (0);
   }
 
   public native void insert (String str, int pos);
@@ -130,11 +160,7 @@ public class GtkTextAreaPeer extends GtkTextComponentPeer
     if (f == null)
       return new Dimension (width, height);
 
-    FontMetrics fm;
-    if (GtkToolkit.useGraphics2D ())
-      fm = new GdkClasspathFontPeerMetrics (f);
-    else
-      fm = new GdkFontMetrics (f);
+    FontMetrics fm = getFontMetrics (f);
 
     int sizeRows = rows == 0 ? DEFAULT_ROWS : rows;
     int sizeCols = cols == 0 ? DEFAULT_COLS : cols;
@@ -163,11 +189,7 @@ public class GtkTextAreaPeer extends GtkTextComponentPeer
     if (f == null)
       return new Dimension (width, height);
 
-    FontMetrics fm;
-    if (GtkToolkit.useGraphics2D ())
-      fm = new GdkClasspathFontPeerMetrics (f);
-    else
-      fm = new GdkFontMetrics (f);
+    FontMetrics fm = getFontMetrics (f);
 
     int sizeRows = rows == 0 ? DEFAULT_ROWS : rows;
     int sizeCols = cols == 0 ? DEFAULT_COLS : cols;
