@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -302,8 +302,12 @@ public final class FileChannelImpl extends FileChannel
   {
     if (!isOpen ())
       throw new ClosedChannelException ();
+
+    force ();
   }
 
+  private native void force ();
+
   // like transferTo, but with a count of less than 2Gbytes
   private int smallTransferTo (long position, int count, 
 			       WritableByteChannel target)
