@@ -16,8 +16,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -40,6 +40,7 @@ package java.net;
 
 import gnu.java.net.PlainDatagramSocketImpl;
 import gnu.java.nio.DatagramChannelImpl;
+
 import java.io.IOException;
 import java.nio.channels.DatagramChannel;
 import java.nio.channels.IllegalBlockingModeException;
@@ -208,7 +209,9 @@ public class DatagramSocket
       }
     catch (IOException e)
       {
-	throw new SocketException(e.getMessage());
+	SocketException se = new SocketException();
+	se.initCause(e);
+	throw se;
       }
   }
 
