@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -38,12 +38,18 @@ exception statement from your version. */
 
 package java.nio;
 
+import gnu.classpath.RawData;
+
+/**
+ * @since 1.4
+ */
 public abstract class Buffer
 {
   int cap = 0;
   int limit = 0;
   int pos = 0;
   int mark = -1;
+  RawData address;
 
   /**
    * Creates a new Buffer.
