@@ -17,8 +17,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -73,14 +73,16 @@ public class SimpleDateFormat extends DateFormat
   private class CompiledField
   {
     /**
-     * The ID of the field within the local pattern characters,
+     * The ID of the field within the local pattern characters.
+     * Package private for use in out class.
      */
-    private int field;
+    int field;
 
     /**
      * The size of the character sequence.
+     * Package private for use in out class.
      */
-    private int size;
+    int size;
 
     /**
      * The character used.
