@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -495,7 +495,8 @@ public class JarFile extends ZipFile
   }
 
   // Only called with lock on this JarFile.
-  private void readSignatures() throws IOException
+  // Package private for use in inner classes.
+  void readSignatures() throws IOException
   {
     Map pkcs7Dsa = new HashMap();
     Map pkcs7Rsa = new HashMap();
@@ -726,7 +727,14 @@ public class JarFile extends ZipFile
     byte[] entryBytes = null;
     try
       {
-        entryBytes = readManifestEntry(super.getEntry(entry));
+	ZipEntry e = super.getEntry(entry);
+	if (e == null)
+	  {
+	    if (DEBUG)
+	      debug("verifyHashes: no entry '" + entry + "'");
+	    return false;
+	  }
+        entryBytes = readManifestEntry(e);
       }
     catch (IOException ioe)
       {
