@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -517,18 +517,7 @@ public abstract class AbstractButton extends JComponent
    */
   public AbstractButton()
   {
-    this("",null);
-  }
-
-  /**
-   * Creates a new AbstractButton object.
-   *
-   * @param txt Value to use for the button's "text" property
-   * @param icon Value to use for the button's "defaultIcon" property
-   */
-  AbstractButton(String txt, Icon icon)
-  {
-    init (txt, icon);
+    init("", null);
     updateUI();
   }
 
@@ -575,7 +564,13 @@ public abstract class AbstractButton extends JComponent
 
  protected void init(String text, Icon icon) 
  {
-    this.text = text;
+    // If text is null, we fall back to the empty
+    // string (which is set using AbstractButton's
+    // constructor).
+    // This way the behavior of the JDK is matched.
+    if(text != null)
+        this.text = text;
+
     default_icon = icon;
     actionListener = createActionListener();
     changeListener = createChangeListener();
@@ -598,24 +593,31 @@ public abstract class AbstractButton extends JComponent
  }
  
   /**
-   * Get the action command string for this button's model.
+   * <p>Returns the action command string for this button's model.</p>
+   *
+   * <p>If the action command was set to <code>null</code>, the button's
+   * text (label) is returned instead.</p>
    *
    * @return The current action command string from the button's model
    */
   public String getActionCommand()
   {
-    return getModel().getActionCommand();
+    String ac = model.getActionCommand();
+    if (ac != null)
+      return ac;
+    else
+      return text;
   }
 
   /**
-   * Set the action command string for this button's model.
+   * Sets the action command string for this button's model.
    *
-   * @param aCommand The new action command string to set in the button's
+   * @param actionCommand The new action command string to set in the button's
    * model.
    */
-  public void setActionCommand(String aCommand)
+  public void setActionCommand(String actionCommand)
   {
-    getModel().setActionCommand(aCommand);
+    model.setActionCommand(actionCommand);
   }
 
   /**
@@ -744,11 +746,19 @@ public abstract class AbstractButton extends JComponent
    */
   protected void fireActionPerformed(ActionEvent e)
   {
-    e.setSource(this);
+	// Dispatch a copy of the given ActionEvent in order to
+	// set the source and action command correctly.
+    ActionEvent ae = new ActionEvent(
+        this,
+        e.getID(),
+        getActionCommand(),
+        e.getWhen(),
+        e.getModifiers());
+
     ActionListener[] listeners = getActionListeners();
     
     for (int i = 0; i < listeners.length; i++)
-      listeners[i].actionPerformed(e);
+      listeners[i].actionPerformed(ae);
   }
 
   /**
@@ -1881,8 +1891,9 @@ public abstract class AbstractButton extends JComponent
     boolean old = contentAreaFilled;
     contentAreaFilled = b;
     firePropertyChange(CONTENT_AREA_FILLED_CHANGED_PROPERTY, old, b);
-    revalidate();
-    repaint();
+    // The JDK sets the opaque property to the value of the contentAreaFilled
+    // property, so should we do.
+    setOpaque(b);
    }
 
   /**
