@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -436,10 +436,12 @@ public class DefaultListSelectionModel implements Cloneable,
     
     // The next if statements breaks down to "if this selection is adjacent
     // to the previous selection and going in the same direction"
-    if (((index0 - 1 == leadSelectionIndex && (index1 >= index0) 
-              && (leadSelectionIndex >= anchorSelectionIndex))
-             || (index0 + 1 == leadSelectionIndex && (index1 <= index0) 
-                 && (leadSelectionIndex <= anchorSelectionIndex)))
+    if ((isSelectedIndex(leadSelectionIndex)) 
+        && ((index0 - 1 == leadSelectionIndex 
+             && (index1 >= index0) 
+             && (leadSelectionIndex >= anchorSelectionIndex))
+            || (index0 + 1 == leadSelectionIndex && (index1 <= index0) 
+                && (leadSelectionIndex <= anchorSelectionIndex)))
         && (anchorSelectionIndex != -1 || leadSelectionIndex != -1))
       {
         // setting setLeadCalledFromAdd to true tells setLeadSelectionIndex
