@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -47,23 +47,23 @@ import javax.swing.event.EventListenerList;
 
 /**
  * DefaultSingleSelectionModel
- * @author	Andrew Selkirk
- * @version	1.0
+ *
+ * @author Andrew Selkirk
  */
 public class DefaultSingleSelectionModel
   implements SingleSelectionModel, Serializable
 {
-  static final long serialVersionUID = 3676229404753786004L;
+  private static final long serialVersionUID = 3676229404753786004L;
 
   /**
    * changeEvent
    */
-  protected transient ChangeEvent changeEvent = new ChangeEvent (this);
+  protected transient ChangeEvent changeEvent = new ChangeEvent(this);
 
   /**
    * listenerList
    */
-  protected EventListenerList listenerList= new EventListenerList ();
+  protected EventListenerList listenerList = new EventListenerList();
 
   /**
    * index
@@ -73,15 +73,16 @@ public class DefaultSingleSelectionModel
   /**
    * Constructor DefaultSingleSelectionModel
    */
-  public DefaultSingleSelectionModel ()
+  public DefaultSingleSelectionModel()
   {
+    // Do nothing.
   }
 
   /**
    * getSelectedIndex
-   * @returns int
+   * @return int
    */
-  public int getSelectedIndex ()
+  public int getSelectedIndex()
   {
     return index;
   }
@@ -90,90 +91,82 @@ public class DefaultSingleSelectionModel
    * setSelectedIndex
    * @param index TODO
    */
-  public void setSelectedIndex (int index)
+  public void setSelectedIndex(int index)
   {
-    // Set Data
     this.index = index;
-
-    // Notify Listeners
-    fireStateChanged ();
+    fireStateChanged();
   }
 
   /**
    * clearSelection
    */
-  public void clearSelection ()
+  public void clearSelection()
   {
-    // Set Data
     index = -1;
-
-    // Notify Listeners
-    fireStateChanged ();
+    fireStateChanged();
   }
 
   /**
    * isSelected
-   * @returns boolean
+   * @return boolean
    */
-  public boolean isSelected ()
+  public boolean isSelected()
   {
-    return (index != -1);
+    return index != -1;
   }
 
   /**
    * addChangeListener
-   * @param listener TODO
+   *
+   * @param listener the listener to add
    */
-  public void addChangeListener (ChangeListener listener)
+  public void addChangeListener(ChangeListener listener)
   {
-    listenerList.add (ChangeListener.class, listener);
+    listenerList.add(ChangeListener.class, listener);
   }
 
   /**
    * removeChangeListener
-   * @param listener TODO
+   *
+   * @param listener the listener to remove
    */
-  public void removeChangeListener (ChangeListener listener)
+  public void removeChangeListener(ChangeListener listener)
   {
-    listenerList.remove (ChangeListener.class, listener);
+    listenerList.remove(ChangeListener.class, listener);
   }
 
   /**
    * fireStateChanged
    */
-  protected void fireStateChanged ()
+  protected void fireStateChanged()
   {
-    // Variables
-    ChangeListener listener;
-    ChangeListener[] listeners;
-    int index;
+    ChangeListener[] listeners = getChangeListeners();
 
-    // Get Listeners
-    listeners = getChangeListeners ();
-
-    // Process Listeners
-    for (index = 0; index < listeners.length; index++)
-      {
-        listener = listeners [index];
-        listener.stateChanged (changeEvent);
-      }
+    for (int i = 0; i < listeners.length; i++)
+      listeners[i].stateChanged(changeEvent);
   }
 
   /**
    * getListeners
-   * @param listenerClass TODO
-   * @returns EventListener[]
+   *
+   * @param listenerClass the type fo listener
+   *
+   * @return an array of listeners
+   *
+   * @since 1.3
    */
-  public EventListener[] getListeners (Class listenerClass)
+  public EventListener[] getListeners(Class listenerClass)
   {
-    return listenerList.getListeners (listenerClass);
+    return listenerList.getListeners(listenerClass);
   }
 
   /**
    * getChangeListeners
+   *
+   * @since 1.4
    */
-  public ChangeListener[] getChangeListeners ()
+  public ChangeListener[] getChangeListeners()
   {
-    return (ChangeListener[]) getListeners (ChangeListener.class);
+    return (ChangeListener[]) getListeners(ChangeListener.class);
   }
 }
