@@ -1,5 +1,5 @@
 /* JButton.java --
-   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -77,9 +77,9 @@ public class JButton extends AbstractButton
 
   public JButton(String text, Icon icon)
   {
-    super(text, icon);
+    super();
+    init(text, icon);
     setModel(new DefaultButtonModel());
-    setActionCommand(text);
   }
 
   public Object[] getSelectedObjects()
@@ -123,7 +123,14 @@ public class JButton extends AbstractButton
 
   protected String paramString()
   {
-    return "JButton";
+    String superParam = super.paramString();
+
+    // 41 is the maximum number of chars which may be needed.
+    StringBuffer sb = new StringBuffer(41);
+    sb.append(",defaultButton=").append(is_def);
+    sb.append(",defaultCapable=").append(def);
+
+    return superParam + sb.toString();
   }
 
   /**
