@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -101,14 +101,14 @@ public class JLabel extends JComponent implements Accessible, SwingConstants
   }
 
   /**
-   * Creates a new vertically centered, horizontally on the leading edge
+   * Creates a new vertically and horizontally centered
    * JLabel object with no text and the given icon.
    *
    * @param image The icon to use with the label.
    */
   public JLabel(Icon image)
   {
-    this(null, image, LEADING);
+    this(null, image, CENTER);
   }
 
   /**
