@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -124,9 +124,18 @@ public class JLayeredPane extends JComponent implements Accessible
    */
   public int getLayer(Component c)
   {
-    if (! componentToLayer.containsKey (c))
-	    throw new IllegalArgumentException ();
-    return ((Integer) componentToLayer.get(c)).intValue();
+    Component myComp = c;
+    while(! componentToLayer.containsKey(myComp))
+      {
+	myComp = myComp.getParent();
+	if (myComp == null)
+	  break;
+      }
+    if (myComp == null)
+      throw new IllegalArgumentException
+	("component is not in this JLayeredPane");
+    Integer layerObj = (Integer) componentToLayer.get(myComp);
+    return layerObj.intValue();
   }
 
   /**
