@@ -1,5 +1,5 @@
 /* JPanel.java --
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -40,9 +40,10 @@ package javax.swing;
 
 import java.awt.FlowLayout;
 import java.awt.LayoutManager;
-import javax.swing.plaf.PanelUI;
+
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
+import javax.swing.plaf.PanelUI;
 
 /**
  * An instance of JPanel can be added to a panel, frame etc
@@ -85,7 +86,7 @@ public class JPanel extends JComponent implements Accessible
     } 
 
     public String getUIClassID()
-    {	return "JPanel";    }
+    {	return "PanelUI";    }
 
 
     public void setUI(PanelUI ui) {
