@@ -1,36 +1,39 @@
-/*
- * JTree.java -- Copyright (C) 2002, 2004, 2005 Free Software Foundation, Inc.
- * 
- * This file is part of GNU Classpath.
- * 
- * GNU Classpath is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2, or (at your option) any later version.
- * 
- * GNU Classpath is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
- * details.
- * 
- * You should have received a copy of the GNU General Public License along with
- * GNU Classpath; see the file COPYING. If not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
- * 
- * Linking this library statically or dynamically with other modules is making a
- * combined work based on this library. Thus, the terms and conditions of the
- * GNU General Public License cover the whole combination.
- * 
- * As a special exception, the copyright holders of this library give you
- * permission to link this library with independent modules to produce an
- * executable, regardless of the license terms of these independent modules, and
- * to copy and distribute the resulting executable under terms of your choice,
- * provided that you also meet, for each linked independent module, the terms
- * and conditions of the license of that module. An independent module is a
- * module which is not derived from or based on this library. If you modify this
- * library, you may extend this exception to your version of the library, but
- * you are not obligated to do so. If you do not wish to do so, delete this
- * exception statement from your version.
- */
+/* JTree.java 
+   Copyright (C) 2002, 2004, 2005 Free Software Foundation, Inc.
+ 
+This file is part of GNU Classpath.
+
+GNU Classpath is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+GNU Classpath is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Classpath; see the file COPYING.  If not, write to the
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
+
+Linking this library statically or dynamically with other modules is
+making a combined work based on this library.  Thus, the terms and
+conditions of the GNU General Public License cover the whole
+combination.
+
+As a special exception, the copyright holders of this library give you
+permission to link this library with independent modules to produce an
+executable, regardless of the license terms of these independent
+modules, and to copy and distribute the resulting executable under
+terms of your choice, provided that you also meet, for each linked
+independent module, the terms and conditions of the license of that
+module.  An independent module is a module which is not derived from
+or based on this library.  If you modify this library, you may extend
+this exception to your version of the library, but you are not
+obligated to do so.  If you do not wish to do so, delete this
+exception statement from your version. */
 
 package javax.swing;
 
@@ -273,7 +276,7 @@ public class JTree
 	 * TreePath of a note to to its state. Valid states are EXPANDED and
 	 * COLLAPSED. Nodes not in this Hashtable are assumed state COLLAPSED.
 	 */
-	private Hashtable nodeStates;
+	private Hashtable nodeStates = new Hashtable();
 	protected transient TreeCellEditor cellEditor;
 	protected transient TreeCellRenderer cellRenderer;
 	protected boolean editable;
@@ -1145,6 +1148,7 @@ public class JTree
 	public void clearSelection()
 	{
 		selectionModel.clearSelection();
+      setLeadSelectionPath(null);
 	}
 
 	public TreePath getLeadSelectionPath()
@@ -1292,7 +1296,7 @@ public class JTree
 		// Don't expand if last path component is a leaf node.
 		if ((path == null) || (treeModel.isLeaf(path.getLastPathComponent())))
 			return;
-
+		
 		setExpandedState(path, true);
 	}
 
@@ -1512,9 +1516,12 @@ public class JTree
 	private void doExpandParents(TreePath path, boolean state)
 	{
 		TreePath parent = path.getParentPath();
-
+		
 		if (isExpanded(parent))
+		{
+			nodeStates.put(path, state ? EXPANDED : COLLAPSED);
 			return;
+		}
 
 		if (parent != null)
 			doExpandParents(parent, false);
@@ -1531,9 +1538,10 @@ public class JTree
 
 		try
 		{
-			while (parent != null)
+			if (parent != null)
 				checkExpandParents(parent);
-		} catch (ExpandVetoException e)
+		} 
+		catch (ExpandVetoException e)
 		{
 			// Expansion vetoed.
 			return;
