@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -87,68 +87,49 @@ public interface DocumentEvent {
 
 	} // ElementChange
 
-	/**
-	 * EventType
-	 */
-	class EventType {
+  /**
+   * EventType
+   */
+  class EventType
+  {
+    /**
+     * INSERT
+     */
+    public static final EventType INSERT = new EventType("INSERT"); // TODO
 
-		//-------------------------------------------------------------
-		// Variables --------------------------------------------------
-		//-------------------------------------------------------------
+    /**
+     * REMOVE
+     */
+    public static final EventType REMOVE = new EventType("REMOVE"); // TODO
 
-		/**
-		 * INSERT
-		 */
-		EventType INSERT = new EventType("INSERT"); // TODO
+    /**
+     * CHANGE
+     */
+    public static final EventType CHANGE = new EventType("CHANGE"); // TODO
 
-		/**
-		 * REMOVE
-		 */
-		EventType REMOVE = new EventType("REMOVE"); // TODO
+    /**
+     * typeString
+     */
+    private String type;
 
-		/**
-		 * CHANGE
-		 */
-		EventType CHANGE = new EventType("CHANGE"); // TODO
+    /**
+     * Constructor EventType
+     * @param type TODO
+     */
+    private EventType(String type)
+    {
+      this.type = type;
+    }
 
-		/**
-		 * typeString
-		 */
-		private String type;
-
-
-		//-------------------------------------------------------------
-		// Initialization ---------------------------------------------
-		//-------------------------------------------------------------
-
-		/**
-		 * Constructor EventType
-		 * @param type TODO
-		 */
-		private EventType(String type) {
-			this.type = type;
-		} // EventType()
-
-
-		//-------------------------------------------------------------
-		// Methods ----------------------------------------------------
-		//-------------------------------------------------------------
-
-		/**
-		 * toString
-		 * @returns String
-		 */
-		public String toString() {
-			return type; // TODO
-		} // toString()
-
-
-	} // EventType
-
-
-	//-------------------------------------------------------------
-	// Methods ----------------------------------------------------
-	//-------------------------------------------------------------
+    /**
+     * toString
+     * @returns String
+     */
+    public String toString()
+    {
+      return type;
+    }
+  }
 
 	/**
 	 * getType
