@@ -1,5 +1,5 @@
 /* SwingPropertyChangeSupport.java --
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -37,10 +37,14 @@ exception statement from your version. */
 
 package javax.swing.event;
 
-// Imports
-import java.beans.*;
-import java.io.*;
-import java.util.*;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
+import java.beans.PropertyChangeSupport;
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
+import java.util.EventListener;
+import java.util.Hashtable;
 
 /**
  * SwingPropertyChangeSupport
@@ -206,8 +210,9 @@ public final	class SwingPropertyChangeSupport
 		PropertyChangeListener	listener;
 
 		// Check Values if they are equal
-		if (event.getOldValue() == null || event.getNewValue() == null ||
-			event.getOldValue().equals(event.getNewValue()) == true) {
+		if (event.getOldValue() == null && event.getNewValue() == null ||
+		    (event.getOldValue() != null && event.getNewValue() != null &&
+	            event.getOldValue().equals(event.getNewValue()))) {
 			return;
 		} // if
 
