@@ -1,5 +1,5 @@
 /* BasicComboBoxEditor.java --
-   Copyright (C) 2004  Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -46,7 +46,6 @@ import java.awt.event.FocusListener;
 import javax.swing.ComboBoxEditor;
 import javax.swing.JTextField;
 import javax.swing.border.EmptyBorder;
-import javax.swing.plaf.UIResource;
 
 /**
  * This is a component that is responsible for displaying/editting  selected
@@ -82,13 +81,13 @@ public class BasicComboBoxEditor extends Object implements ComboBoxEditor,
   }
 
   /**
-   * Sets item that should be editted when any editting operation is performed
+   * Sets item that should be edited when any editing operation is performed
    * by the user. The value is always equal to the currently selected value
    * in the combo box. Thus whenever a different value is selected from the
-   * combo box list then this method should be  called to change editting
+   * combo box list then this method should be  called to change editing
    * item to the new selected item.
    *
-   * @param selectedItem item that is currently selected in the combo box
+   * @param item item that is currently selected in the combo box
    */
   public void setItem(Object item)
   {
