@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -152,8 +152,9 @@ public class BasicFileChooserUI extends FileChooserUI
         {
 	  File f = filechooser.getFileSystemView().createFileObject(obj
 	                                                            .toString());
-	  if (filechooser.isTraversable(f))
-	    filechooser.setCurrentDirectory(f);
+	  if (filechooser.isTraversable(f) && 
+              filechooser.getFileSelectionMode() == JFileChooser.FILES_ONLY)
+            filechooser.setCurrentDirectory(f);
 	  else
 	    {
 	      filechooser.setSelectedFile(f);
@@ -873,76 +874,74 @@ public class BasicFileChooserUI extends FileChooserUI
       }
     };
 
-  // -- begin private --
+  // -- begin private, but package local since used in inner classes --
+
+  JFileChooser filechooser;
 
   /** DOCUMENT ME! */
-  private JFileChooser filechooser;
+  JList filelist;
 
   /** DOCUMENT ME! */
-  private JList filelist;
+  JComboBox filters;
 
   /** DOCUMENT ME! */
-  private JComboBox filters;
+  BasicDirectoryModel model;
 
   /** DOCUMENT ME! */
-  private BasicDirectoryModel model;
+  FileFilter acceptAll = new AcceptAllFileFilter();
 
   /** DOCUMENT ME! */
-  private FileFilter acceptAll = new AcceptAllFileFilter();
+  FileView fv = new BasicFileView();
 
   /** DOCUMENT ME! */
-  private FileView fv = new BasicFileView();
+  static final int ICON_SIZE = 24;
 
   /** DOCUMENT ME! */
-  private static final int ICON_SIZE = 24;
+  JComboBox parents;
 
   /** DOCUMENT ME! */
-  private JComboBox parents;
+  String filename;
 
   /** DOCUMENT ME! */
-  private String filename;
+  JButton accept;
 
   /** DOCUMENT ME! */
-  private JButton accept;
+  JButton cancel;
 
   /** DOCUMENT ME! */
-  private JButton cancel;
+  JButton upFolderButton;
 
   /** DOCUMENT ME! */
-  private JButton upFolderButton;
+  JButton newFolderButton;
 
   /** DOCUMENT ME! */
-  private JButton newFolderButton;
+  JButton homeFolderButton;
 
   /** DOCUMENT ME! */
-  private JButton homeFolderButton;
+  JPanel accessoryPanel;
 
   /** DOCUMENT ME! */
-  private JPanel accessoryPanel;
+  PropertyChangeListener propertyChangeListener;
 
   /** DOCUMENT ME! */
-  private PropertyChangeListener propertyChangeListener;
+  String acceptAllFileFilterText;
 
   /** DOCUMENT ME! */
-  private String acceptAllFileFilterText;
+  String dirDescText;
 
   /** DOCUMENT ME! */
-  private String dirDescText;
+  String fileDescText;
 
   /** DOCUMENT ME! */
-  private String fileDescText;
+  boolean dirSelected = false;
 
   /** DOCUMENT ME! */
-  private boolean dirSelected = false;
+  File currDir = null;
+
+  JPanel bottomPanel;
 
   /** DOCUMENT ME! */
-  private File currDir = null;
-
-  /** DOCUMENT ME! */
-  private JPanel bottomPanel;
-
-  /** DOCUMENT ME! */
-  private JPanel closePanel;
+  JPanel closePanel;
 
   // -- end private --
   private class ListLabelRenderer
@@ -1028,10 +1027,7 @@ public class BasicFileChooserUI extends FileChooserUI
     }
   }
 
-  /**
-   * DOCUMENT ME!
-   */
-  private void closeDialog()
+  void closeDialog()
   {
     Window owner = SwingUtilities.windowForComponent(filechooser);
     if (owner instanceof JDialog)
@@ -1167,10 +1163,7 @@ public class BasicFileChooserUI extends FileChooserUI
       };
   }
 
-  /**
-   * DOCUMENT ME!
-   */
-  private void filterEntries()
+  void filterEntries()
   {
     FileFilter[] list = filechooser.getChoosableFileFilters();
     if (filters.getItemCount() > 0)
