@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -51,6 +51,8 @@ import java.awt.font.TextLayout;
 import java.awt.geom.Rectangle2D;
 
 import javax.swing.AbstractButton;
+import javax.swing.Icon;
+import javax.swing.JComponent;
 import javax.swing.SwingUtilities;
 
 
@@ -585,7 +587,9 @@ public class BasicGraphicsUtils
    *         and <code>height</code> fields indicate the preferred
    *         extent in pixels.
    *
-   * @see javax.swing.SwingUtilities#layoutCompoundLabel
+   * @see javax.swing.SwingUtilities#layoutCompoundLabel(JComponent, 
+   *      FontMetrics, String, Icon, int, int, int, int, Rectangle, Rectangle, 
+   *      Rectangle, int)
    */
   public static Dimension getPreferredButtonSize(AbstractButton b,
                                                  int textIconGap)
