@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -56,6 +56,7 @@ import java.beans.PropertyChangeListener;
 import java.beans.PropertyVetoException;
 import java.beans.VetoableChangeListener;
 
+import javax.swing.BorderFactory;
 import javax.swing.DefaultDesktopManager;
 import javax.swing.DesktopManager;
 import javax.swing.JComponent;
@@ -64,11 +65,16 @@ import javax.swing.JInternalFrame;
 import javax.swing.KeyStroke;
 import javax.swing.SwingConstants;
 import javax.swing.SwingUtilities;
+import javax.swing.UIDefaults;
+import javax.swing.UIManager;
 import javax.swing.border.AbstractBorder;
+import javax.swing.border.BevelBorder;
+import javax.swing.border.Border;
 import javax.swing.event.InternalFrameEvent;
 import javax.swing.event.InternalFrameListener;
 import javax.swing.event.MouseInputAdapter;
 import javax.swing.event.MouseInputListener;
+import javax.swing.plaf.BorderUIResource;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.InternalFrameUI;
 import javax.swing.plaf.UIResource;
@@ -1172,9 +1178,31 @@ public class BasicInternalFrameUI extends InternalFrameUI
    * This method installs the defaults specified by the look and feel.
    */
   protected void installDefaults()
-  {
-    // FIXME: Move border to MetalBorders
-    frame.setBorder(new InternalFrameBorder());
+    {
+      // This is the border of InternalFrames in the BasicLookAndFeel.
+      // Note that there exist entries for various border colors in
+      // BasicLookAndFeel's defaults, but obviously they differ
+      // from the colors that are actually used by the JDK.
+      UIDefaults defaults = UIManager.getLookAndFeelDefaults();
+      Color borderColor = defaults.getColor("InternalFrame.borderColor");
+      Border inner = BorderFactory.createLineBorder(borderColor, 1);
+      Color borderDarkShadow = defaults.getColor
+	  ("InternalFrame.borderDarkShadow");
+      Color borderHighlight = defaults.getColor
+	  ("InternalFrame.borderHighlight");
+      Color borderShadow = defaults.getColor("InternalFrame.borderShadow");
+      Color borderLight = defaults.getColor("InternalFrame.borderLight");
+      Border outer = BorderFactory.createBevelBorder(BevelBorder.RAISED,
+						     borderShadow,
+						     borderHighlight,
+						     borderDarkShadow,
+						     borderShadow);
+      Border border = new BorderUIResource.CompoundBorderUIResource(outer,
+								    inner);
+      frame.setBorder(border);
+
+      // InternalFrames are invisible by default.
+      frame.setVisible(false);
   }
 
   /**
