@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -95,7 +95,7 @@ public class BasicListUI extends ListUI
   }
 
   /**
-   * A helper class which listens for {@link FocusEvents}
+   * A helper class which listens for {@link FocusEvent}s
    * from the JList.
    */
   public class FocusHandler implements FocusListener
@@ -133,7 +133,7 @@ public class BasicListUI extends ListUI
    * A helper class which listens for {@link ListDataEvent}s generated by
    * the {@link JList}'s {@link ListModel}.
    *
-   * @see javax.swing.JList#model
+   * @see javax.swing.JList#getModel()
    */
   public class ListDataHandler implements ListDataListener
   {
@@ -197,10 +197,17 @@ public class BasicListUI extends ListUI
     
     public void keyPressed( KeyEvent evt ) 
     {
-      if (evt.getKeyCode() == KeyEvent.VK_DOWN)
+      int lead = BasicListUI.this.list.getLeadSelectionIndex();
+      int max = BasicListUI.this.list.getModel().getSize() - 1;
+      // Do nothing if list is empty
+      if (max == -1)
+        return;
+
+      // Process the key event.  Bindings can be found in
+      // javax.swing.plaf.basic.BasicLookAndFeel.java
+      if ((evt.getKeyCode() == KeyEvent.VK_DOWN)
+          || (evt.getKeyCode() == KeyEvent.VK_KP_DOWN))
         {
-          int lead = BasicListUI.this.list.getLeadSelectionIndex();
-          int max = BasicListUI.this.list.getModel().getSize() - 1;
           if (!evt.isShiftDown())
             {
               BasicListUI.this.list.clearSelection();
@@ -212,9 +219,9 @@ public class BasicListUI extends ListUI
                 setLeadSelectionIndex(Math.min(lead+1,max));
             }
         }
-      else if (evt.getKeyCode() == KeyEvent.VK_UP)
+      else if ((evt.getKeyCode() == KeyEvent.VK_UP)
+               || (evt.getKeyCode() == KeyEvent.VK_KP_UP))
         {
-          int lead = BasicListUI.this.list.getLeadSelectionIndex();
           if (!evt.isShiftDown())
             {
               BasicListUI.this.list.clearSelection();
@@ -226,6 +233,47 @@ public class BasicListUI extends ListUI
                 setLeadSelectionIndex(Math.max(lead-1,0));
             }
         }
+      else if (evt.getKeyCode() == KeyEvent.VK_PAGE_UP)
+        {
+          // FIXME: implement, need JList.ensureIndexIsVisible to work
+        }
+      else if (evt.getKeyCode() == KeyEvent.VK_PAGE_DOWN)
+        {
+          // FIXME: implement, need JList.ensureIndexIsVisible to work
+        }
+      else if (evt.getKeyCode() == KeyEvent.VK_BACK_SLASH
+               && evt.isControlDown())
+        {
+            BasicListUI.this.list.clearSelection();
+        }
+      else if ((evt.getKeyCode() == KeyEvent.VK_HOME)
+               || evt.getKeyCode() == KeyEvent.VK_END)
+        {
+          // index is either 0 for HOME, or last cell for END
+          int index = (evt.getKeyCode() == KeyEvent.VK_HOME) ? 0 : max;
+          
+          if (!evt.isShiftDown() ||(BasicListUI.this.list.getSelectionMode() 
+                                    == ListSelectionModel.SINGLE_SELECTION))
+            BasicListUI.this.list.setSelectedIndex(index);
+          else if (BasicListUI.this.list.getSelectionMode() == 
+                   ListSelectionModel.SINGLE_INTERVAL_SELECTION)
+            BasicListUI.this.list.setSelectionInterval
+              (BasicListUI.this.list.getAnchorSelectionIndex(), index);
+          else
+            BasicListUI.this.list.getSelectionModel().
+              setLeadSelectionIndex(index);
+        }
+      else if ((evt.getKeyCode() == KeyEvent.VK_A || evt.getKeyCode()
+                == KeyEvent.VK_SLASH) && evt.isControlDown())
+        {
+          BasicListUI.this.list.setSelectionInterval(0, max);
+        }
+      else if (evt.getKeyCode() == KeyEvent.VK_SPACE && evt.isControlDown())
+        {
+          BasicListUI.this.list.getSelectionModel().
+            setLeadSelectionIndex(Math.min(lead+1,max));
+        }
+      
     }
   }
   
@@ -582,8 +630,6 @@ public class BasicListUI extends ListUI
   void damageLayout()
   {
     updateLayoutStateNeeded = 1;
-    if (list != null)
-      list.revalidate();
   }
 
   /**
