@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -47,15 +47,16 @@ import java.io.Serializable;
 import java.util.Enumeration;
 import java.util.ResourceBundle;
 
-import javax.swing.ImageIcon;
+import javax.swing.BorderFactory;
 import javax.swing.KeyStroke;
 import javax.swing.LookAndFeel;
 import javax.swing.UIDefaults;
+import javax.swing.border.BevelBorder;
+import javax.swing.border.Border;
 import javax.swing.plaf.BorderUIResource;
 import javax.swing.plaf.ColorUIResource;
 import javax.swing.plaf.DimensionUIResource;
 import javax.swing.plaf.FontUIResource;
-import javax.swing.plaf.IconUIResource;
 import javax.swing.plaf.InsetsUIResource;
 import javax.swing.text.JTextComponent;
 
@@ -69,7 +70,7 @@ public abstract class BasicLookAndFeel extends LookAndFeel
   static final long serialVersionUID = -6096995660290287879L;
 
   /**
-   * Constructor BasicLookAndFeel
+   * Creates a new instance of the Basic look and feel.
    */
   public BasicLookAndFeel()
   {
@@ -77,8 +78,10 @@ public abstract class BasicLookAndFeel extends LookAndFeel
   }
 
   /**
-   * getDefaults
-   * @return UIDefaults
+   * Creates and returns a new instance of the default resources for this look 
+   * and feel.
+   * 
+   * @return The UI defaults.
    */
   public UIDefaults getDefaults()
   {
@@ -95,8 +98,10 @@ public abstract class BasicLookAndFeel extends LookAndFeel
   }
 
   /**
-   * initClassDefaults
-   * @param value0 TODO
+   * Populates the <code>defaults</code> table with mappings between class IDs 
+   * and fully qualified class names for the UI delegates.
+   * 
+   * @param defaults  the defaults table (<code>null</code> not permitted).
    */
   protected void initClassDefaults(UIDefaults defaults)
   {
@@ -155,8 +160,9 @@ public abstract class BasicLookAndFeel extends LookAndFeel
   }
 
   /**
-   * initSystemColorDefaults
-   * @param defaults TODO
+   * Populates the <code>defaults</code> table with system color defaults.
+   * 
+   * @param defaults  the defaults table (<code>null</code> not permitted).
    */
   protected void initSystemColorDefaults(UIDefaults defaults)
   {
@@ -198,13 +204,14 @@ public abstract class BasicLookAndFeel extends LookAndFeel
   }
 
   /**
-   * loadSystemColors
-   * @param defaults TODO
-   * @param value1 TODO
-   * @param value2 TODO
+   * Loads the system colors.  This method is not implemented yet.
+   * 
+   * @param defaults  the defaults table (<code>null</code> not permitted).
+   * @param systemColors TODO
+   * @param useNative TODO
    */
-  protected void loadSystemColors(UIDefaults defaults, String[] value1,
-                                  boolean value2)
+  protected void loadSystemColors(UIDefaults defaults, String[] systemColors,
+                                  boolean useNative)
   {
     // TODO
   }
@@ -232,7 +239,7 @@ public abstract class BasicLookAndFeel extends LookAndFeel
 
   /**
    * initComponentDefaults
-   * @param defaults TODO
+   * @param defaults  the defaults table (<code>null</code> not permitted).
    */
   protected void initComponentDefaults(UIDefaults defaults)
   {
@@ -247,7 +254,7 @@ public abstract class BasicLookAndFeel extends LookAndFeel
 
       "AbstractUndoableEdit.undoText", "Undo",
       "AbstractUndoableEdit.redoText", "Redo",
-      "Button.background", new ColorUIResource(light),
+      "Button.background", new ColorUIResource(Color.LIGHT_GRAY),
       "Button.border",
       new UIDefaults.LazyValue() 
       {
@@ -256,17 +263,17 @@ public abstract class BasicLookAndFeel extends LookAndFeel
           return BasicBorders.getButtonBorder();
         }
       },
-      "Button.darkShadow", new ColorUIResource(shadow),
+      "Button.darkShadow", new ColorUIResource(Color.BLACK),
       "Button.focusInputMap", new UIDefaults.LazyInputMap(new Object[] {
         "SPACE",  "pressed",
         "released SPACE", "released"
       }),
       "Button.font", new FontUIResource("Dialog", Font.PLAIN, 12),
-      "Button.foreground", new ColorUIResource(darkShadow),
-      "Button.highlight", new ColorUIResource(highLight),
-      "Button.light", new ColorUIResource(highLight),
+      "Button.foreground", new ColorUIResource(Color.BLACK),
+      "Button.highlight", new ColorUIResource(Color.WHITE),
+      "Button.light", new ColorUIResource(Color.LIGHT_GRAY),
       "Button.margin", new InsetsUIResource(2, 2, 2, 2),
-      "Button.shadow", new ColorUIResource(shadow),
+      "Button.shadow", new ColorUIResource(Color.GRAY),
       "Button.textIconGap", new Integer(4),
       "Button.textShiftOffset", new Integer(0),
       "CheckBox.background", new ColorUIResource(light),
@@ -449,12 +456,29 @@ public abstract class BasicLookAndFeel extends LookAndFeel
       "InternalFrame.activeTitleBackground", new ColorUIResource(0, 0, 128),
       "InternalFrame.activeTitleForeground", new ColorUIResource(Color.white),
       "InternalFrame.border",
-      new BorderUIResource.CompoundBorderUIResource(null, null),
+      new UIDefaults.LazyValue()
+      {
+	public Object createValue(UIDefaults table)
+	{
+	  Color lineColor = new Color(238, 238, 238);
+	  Border inner = BorderFactory.createLineBorder(lineColor, 1);
+	  Color shadowInner = new Color(184, 207, 229);
+	  Color shadowOuter = new Color(122, 138, 153);
+	  Border outer = BorderFactory.createBevelBorder(BevelBorder.RAISED,
+							 Color.WHITE,
+							 Color.WHITE,
+							 shadowOuter,
+							 shadowInner);
+	  Border border = new BorderUIResource.CompoundBorderUIResource(outer,
+									inner);
+	  return border;
+	}
+      },
       "InternalFrame.borderColor", new ColorUIResource(light),
-      "InternalFrame.borderDarkShadow", new ColorUIResource(shadow),
-      "InternalFrame.borderHighlight", new ColorUIResource(highLight),
-      "InternalFrame.borderLight", new ColorUIResource(light),
-      "InternalFrame.borderShadow", new ColorUIResource(shadow),
+      "InternalFrame.borderDarkShadow", new ColorUIResource(Color.BLACK),
+      "InternalFrame.borderHighlight", new ColorUIResource(Color.WHITE),
+      "InternalFrame.borderLight", new ColorUIResource(Color.LIGHT_GRAY),
+      "InternalFrame.borderShadow", new ColorUIResource(Color.GRAY),
       "InternalFrame.closeIcon", BasicIconFactory.createEmptyFrameIcon(),
       // XXX Don't use gif
 //      "InternalFrame.icon", new IconUIResource(new ImageIcon("icons/JavaCup.gif")),
@@ -826,7 +850,8 @@ public abstract class BasicLookAndFeel extends LookAndFeel
       "Table.focusCellBackground", new ColorUIResource(light),
       "Table.focusCellForeground", new ColorUIResource(darkShadow),
       "Table.focusCellHighlightBorder",
-      new BorderUIResource.LineBorderUIResource(Color.white),
+      new BorderUIResource.LineBorderUIResource(
+                                             new ColorUIResource(255, 255, 0)),
       "Table.font", new FontUIResource("Dialog", Font.PLAIN, 12),
       "Table.foreground", new ColorUIResource(darkShadow),
       "Table.gridColor", new ColorUIResource(Color.gray),
@@ -838,7 +863,7 @@ public abstract class BasicLookAndFeel extends LookAndFeel
       "TableHeader.font", new FontUIResource("Dialog", Font.PLAIN, 12),
       "TableHeader.foreground", new ColorUIResource(darkShadow),
 
-			"TextArea.background", new ColorUIResource(light),
+            "TextArea.background", new ColorUIResource(light),
       "TextArea.border", new BasicBorders.MarginBorder(),
       "TextArea.caretBlinkRate", new Integer(500),
       "TextArea.caretForeground", new ColorUIResource(Color.black),
@@ -879,11 +904,11 @@ public abstract class BasicLookAndFeel extends LookAndFeel
                                                              0),
                                       "notify-field-accept"),
         new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,
-							     InputEvent.SHIFT_DOWN_MASK),
-							     "selection-backward"),
+                                 InputEvent.SHIFT_DOWN_MASK),
+                                 "selection-backward"),
         new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,
-							     InputEvent.SHIFT_DOWN_MASK),
-							     "selection-forward"),
+                                 InputEvent.SHIFT_DOWN_MASK),
+                                 "selection-forward"),
           },
       "TextField.margin", new InsetsUIResource(0, 0, 0, 0),
       "TextField.selectionBackground", new ColorUIResource(Color.black),
@@ -1011,20 +1036,18 @@ public abstract class BasicLookAndFeel extends LookAndFeel
         "PAGE_UP",  "scrollUpChangeSelection",
         "ctrl PAGE_DOWN", "scrollDownChangeLead"
       }),
-      "Tree.font", new FontUIResource("Dialog", Font.PLAIN, 12),
-//      "Tree.expandedIcon", new IconUIResource(new ImageIcon("icons/TreeExpanded.png")),
+      "Tree.font", new FontUIResource(new Font("Helvetica", Font.PLAIN, 12)),
       "Tree.foreground", new ColorUIResource(Color.black),
-      "Tree.hash", new ColorUIResource(Color.gray),
-//      "Tree.leafIcon", new IconUIResource(new ImageIcon("icons/TreeLeaf.png")),
+      "Tree.hash", new ColorUIResource(new Color(128, 128, 128)),
       "Tree.leftChildIndent", new Integer(7),
-//      "Tree.openIcon", new IconUIResource(new ImageIcon("icons/TreeOpen.png")),
       "Tree.rightChildIndent", new Integer(13),
-      "Tree.rowHeight", new Integer(16),
+      "Tree.rowHeight", new Integer(20), // FIXME
       "Tree.scrollsOnExpand", Boolean.TRUE,
       "Tree.selectionBackground", new ColorUIResource(Color.black),
+      "Tree.nonSelectionBackground", new ColorUIResource(new Color(239, 235, 231)),
       "Tree.selectionBorderColor", new ColorUIResource(Color.black),
-      "Tree.selectionForeground", new ColorUIResource(Color.white),
-      "Tree.textBackground", new ColorUIResource(Color.white),
+      "Tree.selectionForeground", new ColorUIResource(new Color(255, 255, 255)),
+      "Tree.textBackground", new ColorUIResource(new Color(255, 255, 255)),
       "Tree.textForeground", new ColorUIResource(Color.black),
       "Viewport.background", new ColorUIResource(light),
       "Viewport.foreground", new ColorUIResource(Color.black),
