@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -1146,6 +1146,7 @@ public class BasicOptionPaneUI extends OptionPaneUI
     optionPane.setBackground(defaults.getColor("OptionPane.background"));
     optionPane.setForeground(defaults.getColor("OptionPane.foreground"));
     optionPane.setBorder(defaults.getBorder("OptionPane.border"));
+    optionPane.setOpaque(true);
 
     messageBorder = defaults.getBorder("OptionPane.messageAreaBorder");
     messageForeground = defaults.getColor("OptionPane.messageForeground");
