@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -39,18 +39,29 @@ exception statement from your version. */
 package javax.swing.plaf.basic;
 
 import javax.swing.JComponent;
+import javax.swing.JPanel;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.PanelUI;
 
 public class BasicPanelUI extends PanelUI
 {
-    public static ComponentUI createUI(JComponent x) 
-    {
-        return new BasicPanelUI();
-    }
+  public static ComponentUI createUI(JComponent x) 
+  {
+    return new BasicPanelUI();
+  }
 
-    public void installUI(JComponent c)
-    {
-      super.installUI(c);
-    }
+  public void installUI(JComponent c)
+  {
+    super.installUI(c);
+    if (c instanceof JPanel)
+      {
+	JPanel p = (JPanel) c;
+	installDefaults(p);
+      }
+  }
+
+  public void installDefaults(JPanel p)
+  {
+    p.setOpaque(true);
+  }
 }
