@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -50,12 +50,14 @@ import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
 import java.awt.event.MouseEvent;
 
+import javax.swing.BorderFactory;
 import javax.swing.CellRendererPane;
 import javax.swing.JComponent;
 import javax.swing.JTable;
 import javax.swing.ListSelectionModel;
 import javax.swing.UIDefaults;
 import javax.swing.UIManager;
+import javax.swing.border.Border;
 import javax.swing.event.MouseInputListener;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.TableUI;
@@ -77,6 +79,12 @@ public class BasicTableUI
   protected CellRendererPane rendererPane;   
   protected JTable table;
 
+  /** The normal cell border. */
+  Border cellBorder;
+
+  /** The cell border for selected/highlighted cells. */
+  Border highlightCellBorder;
+
   class FocusHandler implements FocusListener
   {
     public void focusGained(FocusEvent e) 
@@ -104,7 +112,7 @@ public class BasicTableUI
   {
     Point begin, curr;
 
-    private void updateSelection()
+    private void updateSelection(boolean controlPressed)
     {
       if (table.getRowSelectionAllowed())
         {
@@ -112,16 +120,29 @@ public class BasicTableUI
           int hi_row  = table.rowAtPoint(curr);
           ListSelectionModel rowModel = table.getSelectionModel();
           if (lo_row != -1 && hi_row != -1)
-            rowModel.setSelectionInterval(lo_row, hi_row);
+            {
+              if (controlPressed && rowModel.getSelectionMode() 
+                  != ListSelectionModel.SINGLE_SELECTION)
+                rowModel.addSelectionInterval(lo_row, hi_row);
+              else
+                rowModel.setSelectionInterval(lo_row, hi_row);
+            }
         }
 
       if (table.getColumnSelectionAllowed())
         {
           int lo_col = table.columnAtPoint(begin);
           int hi_col = table.columnAtPoint(curr);
-          ListSelectionModel colModel = table.getColumnModel().getSelectionModel();
+          ListSelectionModel colModel = table.getColumnModel().
+            getSelectionModel();
           if (lo_col != -1 && hi_col != -1)
-            colModel.setSelectionInterval(lo_col, hi_col);
+            {
+              if (controlPressed && colModel.getSelectionMode() != 
+                  ListSelectionModel.SINGLE_SELECTION)
+                colModel.addSelectionInterval(lo_col, hi_col);
+              else
+                colModel.setSelectionInterval(lo_col, hi_col);
+            }
         }
     }
 
@@ -131,7 +152,7 @@ public class BasicTableUI
     public void mouseDragged(MouseEvent e) 
     {
       curr = new Point(e.getX(), e.getY());
-      updateSelection();      
+      updateSelection(e.isControlDown());      
     }
     public void mouseEntered(MouseEvent e) 
     {
@@ -146,7 +167,20 @@ public class BasicTableUI
     {
       begin = new Point(e.getX(), e.getY());
       curr = new Point(e.getX(), e.getY());
-      updateSelection();
+      //if control is pressed and the cell is already selected, deselect it
+      if (e.isControlDown() && table.
+          isCellSelected(table.rowAtPoint(begin),table.columnAtPoint(begin)))
+        {                                       
+          table.getSelectionModel().
+            removeSelectionInterval(table.rowAtPoint(begin), 
+                                    table.rowAtPoint(begin));
+          table.getColumnModel().getSelectionModel().
+            removeSelectionInterval(table.columnAtPoint(begin), 
+                                    table.columnAtPoint(begin));
+        }
+      else
+        updateSelection(e.isControlDown());
+      
     }
     public void mouseReleased(MouseEvent e) 
     {
@@ -195,6 +229,9 @@ public class BasicTableUI
     table.setSelectionForeground(defaults.getColor("Table.selectionForeground"));
     table.setSelectionBackground(defaults.getColor("Table.selectionBackground"));
     table.setOpaque(true);
+
+    highlightCellBorder = defaults.getBorder("Table.focusCellHighlightBorder");
+    cellBorder = BorderFactory.createEmptyBorder(1, 1, 1, 1);
   }
   protected void installKeyboardActions() 
   {
@@ -205,6 +242,7 @@ public class BasicTableUI
     table.addFocusListener(focusListener);  
     table.addKeyListener(keyListener);
     table.addMouseListener(mouseInputListener);    
+    table.addMouseMotionListener(mouseInputListener);
   }
 
   protected void uninstallDefaults() 
@@ -236,6 +274,7 @@ public class BasicTableUI
     table.removeFocusListener(focusListener);  
     table.removeKeyListener(keyListener);
     table.removeMouseListener(mouseInputListener);    
+    table.removeMouseMotionListener(mouseInputListener);
   }
 
   public void installUI(JComponent comp) 
@@ -292,6 +331,14 @@ public class BasicTableUI
                 Component comp = table.prepareRenderer(rend, r, c);
                 gfx.translate(x, y);
                 comp.setBounds(new Rectangle(0, 0, width, height));
+                // Set correct border on cell renderer.
+                if (comp instanceof JComponent)
+                  {
+                    if (table.isCellSelected(r, c))
+                      ((JComponent) comp).setBorder(highlightCellBorder);
+                    else
+                      ((JComponent) comp).setBorder(cellBorder);
+                  }
                 comp.paint(gfx);
                 gfx.translate(-x, -y);
               }
