@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -208,7 +208,7 @@ public abstract class BasicTextUI extends TextUI
 
   /**
    * Listens for changes on the underlying model and forwards notifications
-   * to the View.
+   * to the View. This also updates the caret position of the text component.
    *
    * TODO: Maybe this should somehow be handled through EditorKits
    */
@@ -236,6 +236,9 @@ public abstract class BasicTextUI extends TextUI
       Dimension size = textComponent.getSize();
       rootView.insertUpdate(ev, new Rectangle(0, 0, size.width, size.height),
                             BasicTextUI.this);
+      int caretPos = textComponent.getCaretPosition();
+      if (caretPos >= ev.getOffset())
+        textComponent.setCaretPosition(caretPos + ev.getLength());
     }
 
     /**
@@ -248,6 +251,9 @@ public abstract class BasicTextUI extends TextUI
       Dimension size = textComponent.getSize();
       rootView.removeUpdate(ev, new Rectangle(0, 0, size.width, size.height),
                             BasicTextUI.this);
+      int caretPos = textComponent.getCaretPosition();
+      if (caretPos >= ev.getOffset())
+        textComponent.setCaretPosition(caretPos - ev.getLength());
     }
   }
 
