@@ -1,37 +1,39 @@
-/*
- * DefaultTreeCellRenderer.java -- Copyright (C) 2002, 2004 Free Software
- * Foundation, Inc.
- * 
- * This file is part of GNU Classpath.
- * 
- * GNU Classpath is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2, or (at your option) any later version.
- * 
- * GNU Classpath is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
- * details.
- * 
- * You should have received a copy of the GNU General Public License along with
- * GNU Classpath; see the file COPYING. If not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
- * 
- * Linking this library statically or dynamically with other modules is making a
- * combined work based on this library. Thus, the terms and conditions of the
- * GNU General Public License cover the whole combination.
- * 
- * As a special exception, the copyright holders of this library give you
- * permission to link this library with independent modules to produce an
- * executable, regardless of the license terms of these independent modules, and
- * to copy and distribute the resulting executable under terms of your choice,
- * provided that you also meet, for each linked independent module, the terms
- * and conditions of the license of that module. An independent module is a
- * module which is not derived from or based on this library. If you modify this
- * library, you may extend this exception to your version of the library, but
- * you are not obligated to do so. If you do not wish to do so, delete this
- * exception statement from your version.
- */
+/* DefaultTreeCellRenderer.java 
+   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
+ 
+This file is part of GNU Classpath.
+
+GNU Classpath is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+GNU Classpath is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Classpath; see the file COPYING.  If not, write to the
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
+
+Linking this library statically or dynamically with other modules is
+making a combined work based on this library.  Thus, the terms and
+conditions of the GNU General Public License cover the whole
+combination.
+
+As a special exception, the copyright holders of this library give you
+permission to link this library with independent modules to produce an
+executable, regardless of the license terms of these independent
+modules, and to copy and distribute the resulting executable under
+terms of your choice, provided that you also meet, for each linked
+independent module, the terms and conditions of the license of that
+module.  An independent module is a module which is not derived from
+or based on this library.  If you modify this library, you may extend
+this exception to your version of the library, but you are not
+obligated to do so.  If you do not wish to do so, delete this
+exception statement from your version. */
 
 package javax.swing.tree;
 
@@ -116,6 +118,7 @@ public class DefaultTreeCellRenderer
 	 * borderSelectionColor
 	 */
 	protected Color borderSelectionColor;
+	
 
 	// -------------------------------------------------------------
 	// Initialization ---------------------------------------------
@@ -135,7 +138,7 @@ public class DefaultTreeCellRenderer
 		setTextNonSelectionColor(defaults.getColor("Tree.textForeground"));
 		setTextSelectionColor(defaults.getColor("Tree.selectionForeground"));
 		setBackgroundNonSelectionColor(defaults
-				.getColor("Tree.textBackground"));
+				.getColor("Tree.nonSelectionBackground"));
 		setBackgroundSelectionColor(defaults
 				.getColor("Tree.selectionBackground"));
 		setBorderSelectionColor(defaults
@@ -376,22 +379,44 @@ public class DefaultTreeCellRenderer
 			boolean selected, boolean expanded, boolean leaf, int row,
 			boolean hasFocus)
 	{
-		this.selected = selected;
-		this.hasFocus = hasFocus;
+		if (val instanceof Icon)
+      	setIcon((Icon) val);
+      else
+      {
+         setText(val.toString());
+         setIcon(null);
+         this.selected = selected;
+         this.hasFocus = hasFocus;
+         setHorizontalAlignment(LEFT);
+         setOpaque(true);
+         setVerticalAlignment(TOP);
+         setEnabled(true);
+         super.setFont(UIManager.getLookAndFeelDefaults().getFont("Tree.font"));
+      }
 
-		if (leaf)
-			setIcon(getLeafIcon());
-		else if (expanded)
-			setIcon(getOpenIcon());
+		if (selected) 
+		{
+			super.setBackground(getBackgroundSelectionColor());
+			setForeground(getTextSelectionColor());
+		}
 		else
-			setIcon(getClosedIcon());
-
-		setText(val.toString());
-		setHorizontalAlignment(LEFT);
-		setVerticalAlignment(TOP);
-
+		{
+			super.setBackground(getBackgroundNonSelectionColor());
+			setForeground(getTextNonSelectionColor());
+		}		
+		
 		return this;
 	}
+	
+	/**
+	 * getFont
+	 * 
+	 * @return the current Font
+	 */
+	public Font getFont()
+	{
+		return super.getFont();
+	}
 
 	/**
 	 * paint
