@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -39,21 +39,25 @@ exception statement from your version. */
 #include "gtkpeer.h"
 #include "gnu_java_awt_peer_gtk_GtkClipboard.h"
 
-jmethodID stringSelectionReceivedID;
-jmethodID stringSelectionHandlerID;
-jmethodID selectionClearID;
+static jmethodID stringSelectionReceivedID;
+static jmethodID stringSelectionHandlerID;
+static jmethodID selectionClearID;
 
-void selection_received (GtkWidget *, GtkSelectionData *, guint, gpointer);
-void selection_get (GtkWidget *, GtkSelectionData *, guint, guint, gpointer);
-gint selection_clear (GtkWidget *, GdkEventSelection *);
+static void selection_received_cb (GtkWidget *, GtkSelectionData *,
+                                   guint, gpointer);
+static void selection_get_cb (GtkWidget *, GtkSelectionData *, guint,
+                              guint, gpointer);
+static gint selection_clear_cb (GtkWidget *, GdkEventSelection *);
 
-GtkWidget *clipboard;
-jobject cb_obj;
+static GtkWidget *clipboard;
+static jobject cb_obj;
 
 JNIEXPORT void JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkClipboard_initNativeState (JNIEnv *env, 
 							 jobject obj)
 {
+  gdk_threads_enter ();
+
   if (!stringSelectionReceivedID)
     {
       jclass gtkclipboard;
@@ -72,20 +76,19 @@ Java_gnu_java_awt_peer_gtk_GtkClipboard_initNativeState (JNIEnv *env,
 
   cb_obj = (*env)->NewGlobalRef (env, obj);
 
-  gdk_threads_enter ();
   clipboard = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   g_signal_connect (G_OBJECT(clipboard), "selection_received",
-		      GTK_SIGNAL_FUNC (selection_received), NULL);
+                    G_CALLBACK (selection_received_cb), NULL);
 
   g_signal_connect (G_OBJECT(clipboard), "selection_clear_event",
-		      GTK_SIGNAL_FUNC (selection_clear), NULL);
+                    G_CALLBACK (selection_clear_cb), NULL);
 
   gtk_selection_add_target (clipboard, GDK_SELECTION_PRIMARY, 
 			    GDK_TARGET_STRING, 0);
 
   g_signal_connect (G_OBJECT(clipboard), "selection_get",
-                      GTK_SIGNAL_FUNC (selection_get), NULL);
+                    G_CALLBACK (selection_get_cb), NULL);
 
   gdk_threads_leave ();
 }
@@ -95,49 +98,63 @@ Java_gnu_java_awt_peer_gtk_GtkClipboard_requestStringConversion
   (JNIEnv *env __attribute__((unused)), jclass clazz __attribute__((unused)))
 {
   gdk_threads_enter ();
+
   gtk_selection_convert (clipboard, GDK_SELECTION_PRIMARY, 
 			 GDK_TARGET_STRING, GDK_CURRENT_TIME);
+
   gdk_threads_leave ();
 }
 
-void
-selection_received (GtkWidget *widget __attribute__((unused)),
-		    GtkSelectionData *selection_data __attribute__((unused)),
-		    guint time __attribute__((unused)),
-		    gpointer data __attribute__((unused)))
+static void
+selection_received_cb (GtkWidget *widget __attribute__((unused)),
+                       GtkSelectionData *selection_data __attribute__((unused)),
+                       guint time __attribute__((unused)),
+                       gpointer data __attribute__((unused)))
 {
   /* Check to see if retrieval succeeded  */
   if (selection_data->length < 0
       || selection_data->type != GDK_SELECTION_TYPE_STRING)
     {
-      (*gdk_env)->CallVoidMethod (gdk_env, cb_obj, stringSelectionReceivedID,
-				  NULL);
+      gdk_threads_leave ();
+
+      (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), cb_obj, stringSelectionReceivedID,
+				    NULL);
+
+      gdk_threads_enter ();
     }
   else
     {
       char *str = (char *) selection_data->data;
       
-      (*gdk_env)->CallVoidMethod (gdk_env, cb_obj, stringSelectionReceivedID,
-				  (*gdk_env)->NewStringUTF (gdk_env, str));
+      gdk_threads_leave ();
+
+      (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), cb_obj, stringSelectionReceivedID,
+				    (*cp_gtk_gdk_env())->NewStringUTF (cp_gtk_gdk_env(), str));
+
+      gdk_threads_enter ();
     }
 
   return;
 }
 
-void
-selection_get (GtkWidget *widget __attribute__((unused)), 
-               GtkSelectionData *selection_data,
-               guint      info __attribute__((unused)),
-               guint      time __attribute__((unused)),
-               gpointer   data __attribute__((unused)))
+static void
+selection_get_cb (GtkWidget *widget __attribute__((unused)), 
+                  GtkSelectionData *selection_data,
+                  guint      info __attribute__((unused)),
+                  guint      time __attribute__((unused)),
+                  gpointer   data __attribute__((unused)))
 {
   jstring jstr;
   const char *utf;
   jsize utflen;
 
-  jstr = (*gdk_env)->CallObjectMethod (gdk_env, cb_obj, 
+  gdk_threads_leave ();
+
+  jstr = (*cp_gtk_gdk_env())->CallObjectMethod (cp_gtk_gdk_env(), cb_obj, 
 				       stringSelectionHandlerID);
 
+  gdk_threads_enter ();
+
   if (!jstr)
     {
       gtk_selection_data_set (selection_data, 
@@ -145,13 +162,13 @@ selection_get (GtkWidget *widget __attribute__((unused)),
       return;
     }
 
-  utflen = (*gdk_env)->GetStringUTFLength (gdk_env, jstr);
-  utf = (*gdk_env)->GetStringUTFChars (gdk_env, jstr, NULL);
+  utflen = (*cp_gtk_gdk_env())->GetStringUTFLength (cp_gtk_gdk_env(), jstr);
+  utf = (*cp_gtk_gdk_env())->GetStringUTFChars (cp_gtk_gdk_env(), jstr, NULL);
 
   gtk_selection_data_set (selection_data, GDK_TARGET_STRING, 8,
 			  (const unsigned char*)utf, utflen);
 
-  (*gdk_env)->ReleaseStringUTFChars (gdk_env, jstr, utf);
+  (*cp_gtk_gdk_env())->ReleaseStringUTFChars (cp_gtk_gdk_env(), jstr, utf);
 }
 
 JNIEXPORT void JNICALL
@@ -173,11 +190,15 @@ Java_gnu_java_awt_peer_gtk_GtkClipboard_selectionGet
   gdk_threads_leave ();
 }
 
-gint
-selection_clear (GtkWidget *widget __attribute__((unused)),
-		 GdkEventSelection *event __attribute__((unused)))
+static gint
+selection_clear_cb (GtkWidget *widget __attribute__((unused)),
+                    GdkEventSelection *event __attribute__((unused)))
 {
-  (*gdk_env)->CallVoidMethod (gdk_env, cb_obj, selectionClearID);
+  gdk_threads_leave ();
+
+  (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), cb_obj, selectionClearID);
+
+  gdk_threads_enter ();
 
   return TRUE;
 }
