@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -40,9 +40,9 @@ exception statement from your version. */
 #include "gnu_java_awt_peer_gtk_GtkComponentPeer.h"
 #include "gnu_java_awt_peer_gtk_GtkFileDialogPeer.h"
 
-static void handle_response (GtkDialog *dialog,
-                             gint responseId,
-                             jobject peer_obj);
+static void handle_response_cb (GtkDialog *dialog,
+                                gint responseId,
+                                jobject peer_obj);
 
 /*
  * Make a new file selection dialog
@@ -55,13 +55,13 @@ Java_gnu_java_awt_peer_gtk_GtkFileDialogPeer_create
   void *parentp;
   gpointer widget;
 
+  gdk_threads_enter ();
+  
   /* Create global reference and save it for future use */
   NSA_SET_GLOBAL_REF (env, obj);
 
   parentp = NSA_GET_PTR(env, parent);
 
-  gdk_threads_enter ();
-  
   /* FIXME: we should be using the default gnome-vfs backend but it is
      not currently thread-safe.  See:
      http://bugzilla.gnome.org/show_bug.cgi?id=166852 */
@@ -79,31 +79,34 @@ Java_gnu_java_awt_peer_gtk_GtkFileDialogPeer_create
 
   /* We must add this window to the group so input in the others are
      disable while it is being shown */
-  gtk_window_group_add_window (global_gtk_window_group, GTK_WINDOW (widget));
-
-  gdk_threads_leave ();
+  gtk_window_group_add_window (cp_gtk_global_window_group,
+                               GTK_WINDOW (widget));
 
   NSA_SET_PTR (env, obj, widget);
+
+  gdk_threads_leave ();
 }
 
 JNIEXPORT void JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkFileDialogPeer_connectSignals
   (JNIEnv *env, jobject obj)
 {
-  void *ptr = NSA_GET_PTR (env, obj);
-  jobject *gref = NSA_GET_GLOBAL_REF (env, obj);
-  g_assert (gref);
+  void *ptr = NULL;
+  jobject *gref = NULL;
 
   gdk_threads_enter ();
 
-  g_signal_connect (G_OBJECT (GTK_DIALOG (ptr)),
-                    "response", 
-		    GTK_SIGNAL_FUNC (handle_response), *gref);
+  ptr = NSA_GET_PTR (env, obj);
+  gref = NSA_GET_GLOBAL_REF (env, obj);
+
+  /* FileDialog signals */
+  g_signal_connect (G_OBJECT (ptr), "response",
+		    G_CALLBACK (handle_response_cb), *gref);
+
+  /* Component signals */
+  cp_gtk_component_connect_signals (G_OBJECT (ptr), gref);
 
   gdk_threads_leave ();
-
-  /* Connect the superclass signals.  */
-  Java_gnu_java_awt_peer_gtk_GtkComponentPeer_connectSignals (env, obj);
 }
 
 JNIEXPORT jstring JNICALL 
@@ -113,10 +116,10 @@ Java_gnu_java_awt_peer_gtk_GtkFileDialogPeer_nativeGetDirectory
   void *ptr;
   const char *str;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
+  ptr = NSA_GET_PTR (env, obj);
+
   str = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER(ptr));
 
   gdk_threads_leave ();
@@ -137,14 +140,16 @@ static gboolean filenameFilterCallback (const GtkFileFilterInfo *filter_info,
   jstring *filename;
   gboolean accepted;
 
-  cx = (*gdk_env())->GetObjectClass (gdk_env(), (jobject) obj);
-  id = (*gdk_env())->GetMethodID (gdk_env(), cx, "filenameFilterCallback",
+  cx = (*cp_gtk_gdk_env())->GetObjectClass (cp_gtk_gdk_env(), (jobject) obj);
+  id = (*cp_gtk_gdk_env())->GetMethodID (cp_gtk_gdk_env(), cx, "filenameFilterCallback",
                                              "(Ljava/lang/String;)Z");
 
-  filename = (*gdk_env())->NewStringUTF(gdk_env(), filter_info->filename);
+  filename = (*cp_gtk_gdk_env())->NewStringUTF(cp_gtk_gdk_env(), filter_info->filename);
 
   gdk_threads_leave();
-  accepted = (*gdk_env())->CallBooleanMethod(gdk_env(), obj, id, filename);
+
+  accepted = (*cp_gtk_gdk_env())->CallBooleanMethod(cp_gtk_gdk_env(), obj, id, filename);
+
   gdk_threads_enter();
 
   return accepted;
@@ -157,10 +162,10 @@ Java_gnu_java_awt_peer_gtk_GtkFileDialogPeer_nativeSetFilenameFilter
   void *ptr;
   GtkFileFilter *filter;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
+  ptr = NSA_GET_PTR (env, obj);
+
   filter = gtk_file_filter_new();
   gtk_file_filter_add_custom(filter, GTK_FILE_FILTER_FILENAME,
 			     filenameFilterCallback, obj, NULL);
@@ -177,15 +182,17 @@ Java_gnu_java_awt_peer_gtk_GtkFileDialogPeer_nativeSetDirectory
   void *ptr;
   const char *str;
 
+  gdk_threads_enter ();
+
   ptr = NSA_GET_PTR (env, obj);
 
   str = (*env)->GetStringUTFChars (env, directory, 0);
 
-  gdk_threads_enter ();
   gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(ptr), str);
-  gdk_threads_leave ();
 
   (*env)->ReleaseStringUTFChars (env, directory, str);
+
+  gdk_threads_leave ();
 }
 
 JNIEXPORT void JNICALL 
@@ -195,21 +202,23 @@ Java_gnu_java_awt_peer_gtk_GtkFileDialogPeer_nativeSetFile
   void *ptr;
   const char *str;
 
+  gdk_threads_enter ();
+
   ptr = NSA_GET_PTR (env, obj);
     
   str = (*env)->GetStringUTFChars (env, filename, 0);
      
-  gdk_threads_enter ();
   gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (ptr), str);
-  gdk_threads_leave ();
 
   (*env)->ReleaseStringUTFChars (env, filename, str);
+
+  gdk_threads_leave ();
 }
 
 static void
-handle_response (GtkDialog *dialog __attribute__((unused)),
-                 gint responseId,
-                 jobject peer_obj)
+handle_response_cb (GtkDialog *dialog __attribute__((unused)),
+                    gint responseId,
+                    jobject peer_obj)
 {
   static int isDisposeIDSet = 0;
   static int isIDSet = 0;
@@ -227,47 +236,45 @@ handle_response (GtkDialog *dialog __attribute__((unused)),
       && responseId != GTK_RESPONSE_CANCEL)
     return;
 
-  ptr = NSA_GET_PTR (gdk_env(), peer_obj);
+  ptr = NSA_GET_PTR (cp_gtk_gdk_env(), peer_obj);
 
   if (responseId == GTK_RESPONSE_DELETE_EVENT)
   {
     if (!isDisposeIDSet)
       {
-        jclass cx = (*gdk_env())->GetObjectClass (gdk_env(), peer_obj);
-        disposeID = (*gdk_env())->GetMethodID (gdk_env(), cx, "gtkDisposeFileDialog", "()V");
+        jclass cx = (*cp_gtk_gdk_env())->GetObjectClass (cp_gtk_gdk_env(), peer_obj);
+        disposeID = (*cp_gtk_gdk_env())->GetMethodID (cp_gtk_gdk_env(), cx, "gtkDisposeFileDialog", "()V");
         isDisposeIDSet = 1;
       }
-  
-    gdk_threads_leave ();
 
     /* We can dispose of the dialog now (and unblock show) */
-    (*gdk_env())->CallVoidMethod (gdk_env(), peer_obj, disposeID);
-
+    gdk_threads_leave ();
+    (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer_obj, disposeID);
     gdk_threads_enter ();
+
     return;
   }
 
   if (responseId == GTK_RESPONSE_ACCEPT) {
     fileName = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (GTK_WIDGET (ptr)));
-    str_fileName = (*gdk_env())->NewStringUTF (gdk_env(), fileName);
+    str_fileName = (*cp_gtk_gdk_env())->NewStringUTF (cp_gtk_gdk_env(), fileName);
   }
 
   if (!isIDSet)
     {
-      jclass cx = (*gdk_env())->GetObjectClass (gdk_env(), peer_obj);
-      hideID = (*gdk_env())->GetMethodID (gdk_env(), cx, "gtkHideFileDialog", "()V");
-      gtkSetFilenameID = (*gdk_env())->GetMethodID (gdk_env(), cx,
+      jclass cx = (*cp_gtk_gdk_env())->GetObjectClass (cp_gtk_gdk_env(), peer_obj);
+      hideID = (*cp_gtk_gdk_env())->GetMethodID (cp_gtk_gdk_env(), cx, "gtkHideFileDialog", "()V");
+      gtkSetFilenameID = (*cp_gtk_gdk_env())->GetMethodID (cp_gtk_gdk_env(), cx,
                                    "gtkSetFilename", "(Ljava/lang/String;)V");
       isIDSet = 1;
     }
-    
-  gdk_threads_leave ();
-  
+
   /* Set the Java object field 'file' with this value. */
-  (*gdk_env())->CallVoidMethod (gdk_env(), peer_obj, gtkSetFilenameID, str_fileName);
+  gdk_threads_leave ();
+  (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer_obj, gtkSetFilenameID, str_fileName);
 
   /* We can hide the dialog now (and unblock show) */
-  (*gdk_env())->CallVoidMethod (gdk_env(), peer_obj, hideID);
+  (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer_obj, hideID);
 
   gdk_threads_enter ();
 }
