@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Classpath; see the file COPYING.  If not, write to the
-   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.
+   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+   02110-1301 USA.
 
    Linking this library statically or dynamically with other modules is
    making a combined work based on this library.  Thus, the terms and
@@ -38,22 +38,34 @@
 #include "gtkpeer.h"
 #include "gnu_java_awt_peer_gtk_GtkListPeer.h"
 
+static jmethodID postListItemEventID;
+
+void
+cp_gtk_list_init_jni (void)
+{
+  jclass gtklistpeer;
+
+  gtklistpeer = (*cp_gtk_gdk_env())->FindClass (cp_gtk_gdk_env(),
+                                         "gnu/java/awt/peer/gtk/GtkListPeer");
+
+  postListItemEventID = (*cp_gtk_gdk_env())->GetMethodID (cp_gtk_gdk_env(), gtklistpeer,
+                                                   "postItemEvent",
+                                                   "(II)V");
+}
+
 enum
   {
     COLUMN_STRING,
     N_COLUMNS
   };
 
-gboolean item_highlighted (GtkTreeSelection *selection,
-                           GtkTreeModel *model,
-                           GtkTreePath *path,
-                           gboolean path_currently_selected,
-                           jobject peer);
+static gboolean item_highlighted_cb (GtkTreeSelection *selection,
+                                     GtkTreeModel *model,
+                                     GtkTreePath *path,
+                                     gboolean path_currently_selected,
+                                     jobject peer);
 
 
-#define TREE_VIEW_FROM_SW(obj) \
-           (GTK_TREE_VIEW (GTK_SCROLLED_WINDOW (obj)->container.child))
-
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GtkListPeer_create
   (JNIEnv *env, jobject obj, jint rows)
@@ -67,11 +79,11 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_create
   GtkRequisition req;
   gint i;
 
+  gdk_threads_enter ();
+
   /* Create global reference and save it for future use */
   NSA_SET_GLOBAL_REF (env, obj);
 
-  gdk_threads_enter ();
-
   list_store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING);
   /* Add the number of rows so that we can calculate the tree view's
      size request. */
@@ -111,23 +123,7 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_create
   gtk_widget_show (list);
   gtk_widget_show (sw);
 
-  gdk_threads_leave ();
-
   NSA_SET_PTR (env, obj, sw);
-}
-
-JNIEXPORT void JNICALL 
-Java_gnu_java_awt_peer_gtk_GtkListPeer_connectJObject
-  (JNIEnv *env, jobject obj)
-{
-  void *ptr;
-
-  ptr = NSA_GET_PTR (env, obj);
-
-  gdk_threads_enter ();
-
-  gtk_widget_realize (GTK_WIDGET (ptr));
-  connect_awt_hook (env, obj, 1, GTK_WIDGET (ptr)->window);
 
   gdk_threads_leave ();
 }
@@ -138,32 +134,27 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_connectSignals
 {
   void *ptr;
   jobject *gref;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreeSelection *selection;
 
+  gdk_threads_enter ();
+
   ptr = NSA_GET_PTR (env, obj);
   gref = NSA_GET_GLOBAL_REF (env, obj);
 
-  gdk_threads_enter ();
-
-  g_assert (gref);
-
-  gtk_widget_realize (GTK_WIDGET (ptr));
-
-  list = TREE_VIEW_FROM_SW (ptr);
-
-  g_signal_connect (G_OBJECT (list), "event",
-                    G_CALLBACK (pre_event_handler), *gref);
+  list = gtk_bin_get_child (GTK_BIN (ptr));
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
-  gtk_tree_selection_set_select_function (selection, item_highlighted,
+  gtk_tree_selection_set_select_function (selection, item_highlighted_cb,
                                           *gref, NULL);
 
+  cp_gtk_component_connect_signals (G_OBJECT (list), gref);
+
   gdk_threads_leave ();
 }
 
 JNIEXPORT void JNICALL
-Java_gnu_java_awt_peer_gtk_GtkListPeer_gtkSetFont
+Java_gnu_java_awt_peer_gtk_GtkListPeer_gtkWidgetModifyFont
   (JNIEnv *env, jobject obj, jstring name, jint style, jint size)
 {
   const char *font_name;
@@ -171,16 +162,17 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_gtkSetFont
   GtkWidget *list;
   PangoFontDescription *font_desc;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter();
 
-  list = GTK_WIDGET (TREE_VIEW_FROM_SW (ptr));
+  ptr = NSA_GET_PTR (env, obj);
+
+  list = gtk_bin_get_child (GTK_BIN (ptr));
 
   font_name = (*env)->GetStringUTFChars (env, name, NULL);
 
   font_desc = pango_font_description_from_string (font_name);
-  pango_font_description_set_size (font_desc, size * dpi_conversion_factor);
+  pango_font_description_set_size (font_desc,
+                                   size * cp_gtk_dpi_conversion_factor);
 
   if (style & AWT_STYLE_BOLD)
     pango_font_description_set_weight (font_desc, PANGO_WEIGHT_BOLD);
@@ -204,11 +196,11 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_gtkWidgetRequestFocus
   void *ptr;
   GtkWidget *list;
 
-  ptr = NSA_GET_PTR (env, obj);
-  
   gdk_threads_enter ();
 
-  list = GTK_WIDGET (TREE_VIEW_FROM_SW (ptr));
+  ptr = NSA_GET_PTR (env, obj);
+  
+  list = gtk_bin_get_child (GTK_BIN (ptr));
   gtk_widget_grab_focus (list);
 
   gdk_threads_leave ();
@@ -219,20 +211,20 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_append
   (JNIEnv *env, jobject obj, jobjectArray items)
 {
   void *ptr;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreeIter iter;
   GtkTreeModel *list_store;
   jint count;
   jint i;
 
+  gdk_threads_enter ();
+
   ptr = NSA_GET_PTR (env, obj);
 
   count = (*env)->GetArrayLength (env, items);
 
-  gdk_threads_enter ();
-
-  list = TREE_VIEW_FROM_SW (ptr);
-  list_store = gtk_tree_view_get_model (list);
+  list = gtk_bin_get_child (GTK_BIN (ptr));
+  list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
   for (i = 0; i < count; i++)
     {
@@ -258,17 +250,17 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_add
 {
   void *ptr;
   const char *str;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreeIter iter;
   GtkTreeModel *list_store;
 
+  gdk_threads_enter ();
+
   ptr = NSA_GET_PTR (env, obj);
   str = (*env)->GetStringUTFChars (env, text, NULL);
 
-  gdk_threads_enter ();
-
-  list = TREE_VIEW_FROM_SW (ptr);
-  list_store = gtk_tree_view_get_model (list);
+  list = gtk_bin_get_child (GTK_BIN (ptr));
+  list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
   if (index == -1)
     gtk_list_store_append (GTK_LIST_STORE (list_store), &iter);
@@ -278,9 +270,9 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_add
   gtk_list_store_set (GTK_LIST_STORE (list_store), &iter,
                       COLUMN_STRING, str, -1);
 
-  gdk_threads_leave ();
-
   (*env)->ReleaseStringUTFChars (env, text, str);
+
+  gdk_threads_leave ();
 }
 
 
@@ -289,18 +281,18 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_delItems
   (JNIEnv *env, jobject obj, jint start, jint end)
 {
   void *ptr;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreeIter iter;
   GtkTreeModel *list_store;
   jint i;
   jint num_items;
     
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
-  list = TREE_VIEW_FROM_SW (ptr);
-  list_store = gtk_tree_view_get_model (list);
+  ptr = NSA_GET_PTR (env, obj);
+
+  list = gtk_bin_get_child (GTK_BIN (ptr));
+  list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
   /* Special case: remove all rows. */
   if (end == -1)
@@ -325,16 +317,16 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_select
   (JNIEnv *env, jobject obj, jint index)
 {
   void *ptr;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreePath *path;
     
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
-  list = TREE_VIEW_FROM_SW (ptr);
+  ptr = NSA_GET_PTR (env, obj);
+
+  list = gtk_bin_get_child (GTK_BIN (ptr));
   path = gtk_tree_path_new_from_indices (index, -1);
-  gtk_tree_view_set_cursor (list, path, NULL, FALSE);
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (list), path, NULL, FALSE);
 
   gdk_threads_leave ();
 }
@@ -344,16 +336,16 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_deselect
   (JNIEnv *env, jobject obj, jint index)
 {
   void *ptr;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreeSelection *selection;
   GtkTreePath *path;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
-  list = TREE_VIEW_FROM_SW (ptr);
-  selection = gtk_tree_view_get_selection (list);
+  ptr = NSA_GET_PTR (env, obj);
+
+  list = gtk_bin_get_child (GTK_BIN (ptr));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
   path = gtk_tree_path_new_from_indices (index, -1);
   gtk_tree_selection_unselect_path (selection, path);
 
@@ -369,13 +361,13 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_getSize
   GtkRequisition current_req;
   GtkRequisition natural_req;
 
+  gdk_threads_enter ();
+
   dims = (*env)->GetIntArrayElements (env, jdims, NULL);
   dims[0] = dims[1] = 0;
 
   ptr = NSA_GET_PTR (env, obj);
 
-  gdk_threads_enter ();
-
   /* Save the widget's current size request. */
   gtk_widget_size_request (GTK_WIDGET (ptr), &current_req);
       
@@ -399,9 +391,9 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_getSize
   else
     dims[1] = natural_req.height / visible_rows * rows;
 
-  gdk_threads_leave ();
-
   (*env)->ReleaseIntArrayElements (env, jdims, dims, 0);
+
+  gdk_threads_leave ();
 }
 
 
@@ -410,7 +402,7 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_getSelectedIndexes
   (JNIEnv *env, jobject obj)
 {
   void *ptr;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreeSelection *selection;
   jintArray result_array;
   jint *result_array_iter;
@@ -420,18 +412,23 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_getSelectedIndexes
   jint count;
   jint i;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
-  list = TREE_VIEW_FROM_SW (ptr);
-  selection = gtk_tree_view_get_selection (list);
+  ptr = NSA_GET_PTR (env, obj);
+
+  list = gtk_bin_get_child (GTK_BIN (ptr));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
   count = gtk_tree_selection_count_selected_rows (selection);
   if (count > 0)
     {
       current_row = rows = gtk_tree_selection_get_selected_rows (selection, NULL);
 
+      gdk_threads_leave ();
+
       result_array = (*env)->NewIntArray (env, count);
+
+      gdk_threads_enter ();
+
       result_array_iter = (*env)->GetIntArrayElements (env, result_array, NULL);
 
       for (i = 0; i < count; i++)
@@ -447,16 +444,14 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_getSelectedIndexes
           g_list_free (rows);
         }
 
-      gdk_threads_leave ();
-
       (*env)->ReleaseIntArrayElements (env, result_array, result_array_iter, 0);
-
-      return result_array;
     }
   else
-    gdk_threads_leave ();
+    result_array = NULL;
 
-  return (jintArray) NULL;
+  gdk_threads_leave ();
+
+  return result_array;
 }
 
 JNIEXPORT void JNICALL
@@ -464,16 +459,17 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_makeVisible
   (JNIEnv *env, jobject obj, jint index)
 {
   void *ptr;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreePath *path;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
-  list = TREE_VIEW_FROM_SW (ptr);
+  ptr = NSA_GET_PTR (env, obj);
+
+  list = gtk_bin_get_child (GTK_BIN (ptr));
   path = gtk_tree_path_new_from_indices (index, -1);
-  gtk_tree_view_scroll_to_cell (list, path, NULL, FALSE, 0.0, 0.0);
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list), path,
+                                NULL, FALSE, 0.0, 0.0);
 
   gdk_threads_leave ();
 }
@@ -483,15 +479,15 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_setMultipleMode
   (JNIEnv *env, jobject obj, jboolean mode)
 {
   void *ptr;
-  GtkTreeView *list;
+  GtkWidget *list;
   GtkTreeSelection *selection;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
 
-  list = TREE_VIEW_FROM_SW (ptr);
-  selection = gtk_tree_view_get_selection (list);
+  ptr = NSA_GET_PTR (env, obj);
+
+  list = gtk_bin_get_child (GTK_BIN (ptr));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
   gtk_tree_selection_set_mode (selection,
                                mode ? GTK_SELECTION_MULTIPLE
                                : GTK_SELECTION_SINGLE);
@@ -499,12 +495,12 @@ Java_gnu_java_awt_peer_gtk_GtkListPeer_setMultipleMode
   gdk_threads_leave ();
 }
 
-gboolean
-item_highlighted (GtkTreeSelection *selection __attribute__((unused)),
-                  GtkTreeModel *model,
-                  GtkTreePath *path,
-                  gboolean path_currently_selected,
-                  jobject peer)
+static gboolean
+item_highlighted_cb (GtkTreeSelection *selection __attribute__((unused)),
+                     GtkTreeModel *model,
+                     GtkTreePath *path,
+                     gboolean path_currently_selected,
+                     jobject peer)
 {
   GtkTreeIter iter;
   jint row;
@@ -516,15 +512,27 @@ item_highlighted (GtkTreeSelection *selection __attribute__((unused)),
       row = indices ? indices[0] : -1;
 
       if (!path_currently_selected)
-        (*gdk_env)->CallVoidMethod (gdk_env, peer,
-                                    postListItemEventID,
-                                    row,
-                                    (jint) AWT_ITEM_SELECTED);
+        {
+          gdk_threads_leave ();
+
+          (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer,
+                                        postListItemEventID,
+                                        row,
+                                        (jint) AWT_ITEM_SELECTED);
+
+          gdk_threads_enter ();
+        }
       else
-        (*gdk_env)->CallVoidMethod (gdk_env, peer,
-                                    postListItemEventID,
-                                    row,
-                                    (jint) AWT_ITEM_DESELECTED);
+        {
+          gdk_threads_leave ();
+
+          (*cp_gtk_gdk_env())->CallVoidMethod (cp_gtk_gdk_env(), peer,
+                                        postListItemEventID,
+                                        row,
+                                        (jint) AWT_ITEM_DESELECTED);
+
+          gdk_threads_enter ();
+        }
     }
 
   return TRUE;
