@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -39,48 +39,80 @@ exception statement from your version. */
 #include "gtkpeer.h"
 #include "gnu_java_awt_peer_gtk_GtkMenuBarPeer.h"
 
-JNIEXPORT void JNICALL Java_gnu_java_awt_peer_gtk_GtkMenuBarPeer_create
+JNIEXPORT void JNICALL
+Java_gnu_java_awt_peer_gtk_GtkMenuBarPeer_create
   (JNIEnv *env, jobject obj)
 {
   GtkWidget *widget;
 
-  /* Create global reference and save it for future use */
-  NSA_SET_GLOBAL_REF (env, obj);
-
   gdk_threads_enter ();
   
+  NSA_SET_GLOBAL_REF (env, obj);
+
   widget = gtk_menu_bar_new ();
   gtk_widget_show (widget);
 
-  gdk_threads_leave ();
-
   NSA_SET_PTR (env, obj, widget);
+
+  gdk_threads_leave ();
 }
 
-JNIEXPORT void JNICALL Java_gnu_java_awt_peer_gtk_GtkMenuBarPeer_addMenu
+JNIEXPORT void JNICALL
+Java_gnu_java_awt_peer_gtk_GtkMenuBarPeer_addMenu
   (JNIEnv *env, jobject obj, jobject menupeer)
 {
   void *mbar, *menu;
 
+  gdk_threads_enter ();
+
   mbar = NSA_GET_PTR (env, obj);
   menu = NSA_GET_PTR (env, menupeer);
 
-  gdk_threads_enter ();
   gtk_menu_shell_append (GTK_MENU_SHELL (mbar), GTK_WIDGET (menu));
+
   gdk_threads_leave ();
 }
 
-JNIEXPORT void JNICALL Java_gnu_java_awt_peer_gtk_GtkMenuBarPeer_delMenu
+JNIEXPORT void JNICALL
+Java_gnu_java_awt_peer_gtk_GtkMenuBarPeer_nativeSetHelpMenu
+  (JNIEnv *env, jobject obj, jobject menupeer)
+{
+  static void *helpmenu;
+  void *mbar, *menu;
+  GList *list;
+
+  gdk_threads_enter ();
+
+  mbar = NSA_GET_PTR (env, obj);
+  menu = NSA_GET_PTR (env, menupeer);
+
+  if (helpmenu != NULL)
+    {
+      list = gtk_container_children (GTK_CONTAINER (mbar));
+      while (list != NULL && list->data != helpmenu)
+        list = list->next;
+      if (list != NULL && list->data == helpmenu)
+        gtk_container_remove (GTK_CONTAINER (mbar), GTK_WIDGET (list->data));
+    }
+  helpmenu = menu;
+
+  gdk_threads_leave ();
+}
+
+JNIEXPORT void JNICALL
+Java_gnu_java_awt_peer_gtk_GtkMenuBarPeer_delMenu
   (JNIEnv *env, jobject obj, jint index)
 {
   void *ptr;
   GList *list;
 
+  gdk_threads_enter ();
+
   ptr = NSA_GET_PTR (env, obj);
 
-  gdk_threads_enter ();
   list = gtk_container_children (GTK_CONTAINER (ptr));
   list = g_list_nth (list, index);
   gtk_container_remove (GTK_CONTAINER (ptr), GTK_WIDGET (list->data));
+
   gdk_threads_leave ();
 }
