@@ -1,5 +1,5 @@
 /* gnu_java_nio_channels_FileChannelImpl.c -
-   Copyright (C) 2003, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -52,6 +52,10 @@ exception statement from your version. */
 
 #include "gnu_java_nio_channels_FileChannelImpl.h"
 
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+
 /* These values must be kept in sync with FileChannelImpl.java.  */
 #define FILECHANNELIMPL_READ   1
 #define FILECHANNELIMPL_WRITE  2
@@ -539,7 +543,7 @@ Java_gnu_java_nio_channels_FileChannelImpl_read__ (JNIEnv * env, jobject obj)
 	  return (-1);
 	}
     }
-  while (bytes_read != 1);
+  while (result != TARGET_NATIVE_OK);
 
   return ((jint) (data & 0xFF));
 }
@@ -567,6 +571,12 @@ Java_gnu_java_nio_channels_FileChannelImpl_read___3BII (JNIEnv * env,
   if (length == 0)
     return 0;
 
+  if (offset < 0)
+    {
+      JCL_ThrowException (env, IO_EXCEPTION, "negative offset");
+      return -1;
+    }
+
   bufptr = (*env)->GetByteArrayElements (env, buffer, 0);
   if (!bufptr)
     {
@@ -574,6 +584,13 @@ Java_gnu_java_nio_channels_FileChannelImpl_read___3BII (JNIEnv * env,
       return (-1);
     }
 
+  if (length + offset > (*env)->GetArrayLength (env, buffer))
+    {
+      JCL_ThrowException (env, IO_EXCEPTION,
+			  "length + offset > buffer.length");
+      return -1;
+    }
+
   bytes_read = 0;
   do
     {
@@ -596,7 +613,8 @@ Java_gnu_java_nio_channels_FileChannelImpl_read___3BII (JNIEnv * env,
 	  (*env)->ReleaseByteArrayElements (env, buffer, bufptr, 0);
 	  return -1;
 	}
-      bytes_read += n;
+      if (result == TARGET_NATIVE_OK)
+	bytes_read += n;
     }
   while (bytes_read < 1);
 
@@ -630,11 +648,28 @@ Java_gnu_java_nio_channels_FileChannelImpl_write__I (JNIEnv * env,
 	{
 	  JCL_ThrowException (env, IO_EXCEPTION,
 			      TARGET_NATIVE_LAST_ERROR_STRING ());
+	  return;
 	}
     }
   while (result != TARGET_NATIVE_OK);
 }
 
+/*
+ * Copies all parts of a file to disk.
+ */
+JNIEXPORT void JNICALL
+Java_gnu_java_nio_channels_FileChannelImpl_force (JNIEnv * env,
+						  jobject obj)
+{
+  int native_fd;
+  int result;
+  native_fd = get_native_fd (env, obj);
+  TARGET_NATIVE_FILE_FSYNC (native_fd, result);
+  if (result != TARGET_NATIVE_OK)
+    JCL_ThrowException (env, IO_EXCEPTION,
+			TARGET_NATIVE_LAST_ERROR_STRING ());
+}
+
 /*
  * Writes a byte buffer to the specified file descriptor
  * Return status code, exception on error
@@ -679,35 +714,80 @@ Java_gnu_java_nio_channels_FileChannelImpl_write___3BII (JNIEnv * env,
 	  (*env)->ReleaseByteArrayElements (env, buffer, bufptr, 0);
 	  return;
 	}
-      bytes_written += n;
+      if (result == TARGET_NATIVE_OK)
+	bytes_written += n;
     }
 
   (*env)->ReleaseByteArrayElements (env, buffer, bufptr, 0);
 }
 
 JNIEXPORT jboolean JNICALL
-Java_gnu_java_nio_channels_FileChannelImpl_lock (JNIEnv * env,
-						 jobject obj
-						 __attribute__ ((__unused__)),
-						 jlong position
-						 __attribute__ ((__unused__)),
-						 jlong size
-						 __attribute__ ((__unused__)),
-						 jboolean shared
-						 __attribute__ ((__unused__)),
-						 jboolean wait
-						 __attribute__ ((__unused__)))
+Java_gnu_java_nio_channels_FileChannelImpl_lock (JNIEnv *env, jobject obj,
+                                                 jlong position, jlong size,
+                                                 jboolean shared, jboolean wait)
 {
-  JCL_ThrowException (env, IO_EXCEPTION,
-		      "java.nio.FileChannelImpl.lock(): not implemented");
-  return 0;
+#ifdef HAVE_FCNTL
+  int fd = get_native_fd (env, obj);
+  int cmd = wait ? F_SETLKW : F_SETLK;
+  struct flock flock;
+  int ret;
+
+  flock.l_type = shared ? F_RDLCK : F_WRLCK;
+  flock.l_whence = SEEK_SET;
+  flock.l_start = (off_t) position;
+  flock.l_len = (off_t) size;
+
+  ret = fcntl (fd, cmd, &flock);
+  if (ret)
+    {
+      /* Linux man pages for fcntl state that errno might be either
+         EACCES or EAGAIN if we try F_SETLK, and another process has
+         an overlapping lock. */
+      if (errno != EACCES && errno != EAGAIN)
+        {
+          JCL_ThrowException (env, IO_EXCEPTION, strerror (errno));
+        }
+      return JNI_FALSE;
+    }
+  return JNI_TRUE;
+#else
+  (void) obj;
+  (void) position;
+  (void) size;
+  (void) shared;
+  (void) wait;
+  JCL_ThrowException (env, "java/lang/UnsupportedOperationException",
+                      "file locks not implemented on this platform");
+  return JNI_FALSE;
+#endif /* HAVE_FCNTL */
 }
 
 JNIEXPORT void JNICALL
-Java_gnu_java_nio_channels_FileChannelImpl_unlock (JNIEnv * env,
-						   jobject obj
-						   __attribute__ ((__unused__)), jlong position __attribute__ ((__unused__)), jlong length __attribute__ ((__unused__)))
+Java_gnu_java_nio_channels_FileChannelImpl_unlock (JNIEnv *env,
+                                                   jobject obj,
+                                                   jlong position,
+                                                   jlong length)
 {
-  JCL_ThrowException (env, IO_EXCEPTION,
-		      "java.nio.FileChannelImpl.unlock(): not implemented");
+#ifdef HAVE_FCNTL
+  int fd = get_native_fd (env, obj);
+  struct flock flock;
+  int ret;
+
+  flock.l_type = F_UNLCK;
+  flock.l_whence = SEEK_SET;
+  flock.l_start = (off_t) position;
+  flock.l_len = (off_t) length;
+
+  ret = fcntl (fd, F_SETLK, &flock);
+  if (ret)
+    {
+      JCL_ThrowException (env, IO_EXCEPTION, strerror (errno));
+    }
+#else
+  (void) obj;
+  (void) position;
+  (void) length;
+  JCL_ThrowException (env, "java/lang/UnsupportedOperationException",
+                      "file locks not implemented on this platform");
+#endif /* HAVE_FCNTL */
 }
